package com.alienfactory.javamappy;

import com.alienfactory.javamappy.util.ParameterChecker;



// -----------------------------------------------------------------------------
/**
 * AnimBlock represents a sequence of frames (images) that will be played out
 * according to the animation type and a speed dictated by the frame delay. <br>
 * <br>
 * The animation type names and sequence events are true to the Mappy code. (I
 * hope!) <br>
 *
 * @author Steve Eynon
 **/
// -----------------------------------------------------------------------------
public class AnimBlock {
	private int type;
	private int delay;
	private int delayCountdown;
	private int userData;
	private int currentFrameIndex;
	private int[] frames = new int[0];

	/**
	 * Do Not Animate. An <code>AnimBlock</code> type. <br>
	 * <br>
	 * By setting the animation type to AN_NONE the current frame will stay as
	 * it is. <br>
	 * <br>
	 * Given 5 frames and no delay, upon successive calls to
	 * {@link #updateAnimation()}, {@link #getCurrentFrameIndex()} would
	 * return: <br>
	 * <br>
	 * <code>[ 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 . . . ]</code> <br>
	 * <br>
	 * Has a constant value of <code>0</code>. <br>
	 * @see #getType()
	 * @see #setType(int)
	 */
    public static final int AN_NONE 	= 0;



	/**
	 * Loop Forward. An <code>AnimBlock</code> type. <br>
	 * <br>
	 * Given 5 frames and no delay, upon successive calls to
	 * {@link #updateAnimation()}, {@link #getCurrentFrameIndex()} would
	 * return: <br>
	 * <br>
	 * <code>[ 0 1 2 3 4 0 1 2 3 4 0 1 2 3 4 0 1 3 . . . ]</code> <br>
	 * <br>
	 * Has a constant value of <code>1</code>. <br>
	 * @see #getType()
	 * @see #setType(int)
	 */
    public static final int AN_LOOPF 	= 1;



	/**
	 * Loop Reverse. An <code>AnimBlock</code> type. <br>
	 * <br>
	 * Given 5 frames and no delay, upon successive calls to
	 * {@link #updateAnimation()}, {@link #getCurrentFrameIndex()} would
	 * return: <br>
	 * <br>
	 * <code>[ 4 3 2 1 0 4 3 2 1 0 4 3 2 1 0 4 3 2 . . . ]</code> <br>
	 * <br>
	 * Has a constant value of <code>2</code>. <br>
	 * @see #getType()
	 * @see #setType(int)
	 */
    public static final int AN_LOOPR 	= 2;



	/**
	 * Animate Once. An <code>AnimBlock</code> type. <br>
	 * <br>
	 * After the animation sequence finishes the current frame will reset to the
	 * first frame and the animation type will change to {@link #AN_ONCES}. <br>
	 * <br>
	 * Given 5 frames and no delay, upon successive calls to
	 * {@link #updateAnimation()}, {@link #getCurrentFrameIndex()} would
	 * return: <br>
	 * <br>
	 * <code>[ 0 1 2 3 4 0 0 0 0 0 0 0 0 0 0 0 0 0 . . . ]</code> <br>
	 * <br>
	 * Has a constant value of <code>3</code>. <br>
	 * @see #getType()
	 * @see #setType(int)
	 */
    public static final int AN_ONCE 	= 3;



    /**
	 * Animate Once Finished. An <code>AnimBlock</code> type. <br>
	 * <br>
	 * The animation type is automatically set to this after the {@link #AN_ONCE}
	 * sequence has finished. <code>AN_ONCES</code> behaves the same as
	 * {@link #AN_ONCES}. <br>
	 * <br>
	 * Has a constant value of <code>9</code>. <br>
	 * @see #getType()
	 * @see #setType(int)
	 */
    public static final int AN_ONCES 	= 9;



	/**
	 * Animate Once & Halt. An <code>AnimBlock</code> type. <br>
	 * <br>
	 * After the animation sequence finishes the current frame sticks to the
	 * last frame. Unlike {@link #AN_ONCE} the animation type does not change.
	 * <br>
	 * <br>
	 * Given 5 frames and no delay, upon successive calls to
	 * {@link #updateAnimation()}, {@link #getCurrentFrameIndex()} would
	 * return: <br>
	 * <br>
	 * <code>[ 0 1 2 3 4 4 4 4 4 4 4 4 4 4 4 4 4 4 . . . ]</code> <br>
	 * <br>
	 * Has a constant value of <code>4</code>. <br>
	 * @see #getType()
	 * @see #setType(int)
	 */
    public static final int AN_ONCEH 	= 4;



	/**
	 * Ping Ping Forward - Outward Leg. An <code>AnimBlock</code> type. <br>
	 * <br>
	 * When the animation sequence reaches the end of the forward 'ping' the
	 * animation type changes to {@link #AN_PPFR} and the sequence begins it's
	 * return journey. <br>
	 * <br>
	 * It could be argued that there is no difference between
	 * <code>AN_PPFF</code> and {@link #AN_PPRR}. <br>
	 * <br>
	 * Given 5 frames and no delay, upon successive calls to
	 * {@link #updateAnimation()}, {@link #getCurrentFrameIndex()} would
	 * return: <br>
	 * <br>
	 * <code>[ 0 1 2 3 4 3 2 1 0 1 2 3 4 3 2 1 0 1 . . . ]</code> <br>
	 * <br>
	 * Has a constant value of <code>5</code>. <br>
	 * @see #getType()
	 * @see #setType(int)
	 */
    public static final int AN_PPFF 	= 5;



	/**
	 * Ping Ping Forward - Return Leg. An <code>AnimBlock</code> type. <br>
	 * <br>
	 * When the animation sequence reaches the end of the forward 'pong' the
	 * animation type changes to {@link #AN_PPFF} and the sequence starts all
	 * over again. <br>
	 * <br>
	 * It could be argued that there is no difference between
	 * <code>AN_PPFR</code> and {@link #AN_PPRF}. <br>
	 * <br>
	 * Given 5 frames and no delay, upon successive calls to
	 * {@link #updateAnimation()}, {@link #getCurrentFrameIndex()} would
	 * return: <br>
	 * <br>
	 * <code>[ 4 3 2 1 0 1 2 3 4 3 2 1 0 1 2 3 4 3 . . . ]</code> <br>
	 * <br>
	 * Has a constant value of <code>6</code>. <br>
	 * @see #getType()
	 * @see #setType(int)
	 */
    public static final int AN_PPFR 	= 6;



	/**
	 * Ping Ping Reverse - Outward Leg. An <code>AnimBlock</code> type. <br>
	 * <br>
	 * When the animation sequence reaches the end of the forward 'ping' the
	 * animation type changes to {@link #AN_PPRR} and the sequence begins it's
	 * return journey. <br>
	 * <br>
	 * It could be argued that there is no difference between
	 * <code>AN_PPRF</code> and {@link #AN_PPFR}. <br>
	 * <br>
	 * Given 5 frames and no delay, upon successive calls to
	 * {@link #updateAnimation()}, {@link #getCurrentFrameIndex()} would
	 * return: <br>
	 * <br>
	 * <code>[ 4 3 2 1 0 1 2 3 4 3 2 1 0 1 2 3 4 3 . . . ]</code> <br>
	 * <br>
	 * Has a constant value of <code>7</code>. <br>
	 * @see #getType()
	 * @see #setType(int)
	 */
    public static final int AN_PPRF 	= 7;



	/**
	 * Ping Ping Reverse - Return Leg. An <code>AnimBlock</code> type. <br>
	 * <br>
	 * When the animation sequence reaches the end of the forward 'pong' the
	 * animation type changes to {@link #AN_PPRR} and the sequence starts all
	 * over again. <br>
	 * <br>
	 * It could be argued that there is no difference between
	 * <code>AN_PPRR</code> and {@link #AN_PPFF}. <br>
	 * <br>
	 * Given 5 frames and no delay, upon successive calls to
	 * {@link #updateAnimation()}, {@link #getCurrentFrameIndex()} would
	 * return: <br>
	 * <br>
	 * <code>[ 0 1 2 3 4 3 2 1 0 1 2 3 4 3 2 1 0 1 . . . ]</code> <br>
	 * <br>
	 * Has a constant value of <code>8</code>. <br>
	 * @see #getType()
	 * @see #setType(int)
	 */
    public static final int AN_PPRR 	= 8;



    /**
     * The <code>type</code> dictates how the animation sequence will be played
     * out. Will it play once and stop or loop forever? <br>
     * @return	the animation type
     */
	public int getType() {
		return type;
	}



    /**
     * The <code>type</code> dictates how the animation sequence will be played
     * out. Will it play once and stop or loop forever? <br>
     * <br>
     * Setting the animation type does not reset / any other values. <br>
     * @param	type the new animation type
     * @throws IllegalArgumentException if <code>type</code> is out of bounds
     */
	public void setType(int type) throws IllegalArgumentException {
		ParameterChecker.checkMinMax(type, "type", AN_NONE, AN_ONCES);
		this.type = type;
	}



    /**
     * The <code>delay</code> is the number of times {@link #updateAnimation()}
     * needs to be called before the <code>currentFrameIndex</code> changes. In
     * effect this controls the speed of the animation. <br>
     * @return the frame delay
     * @see #getDelayCountdown()
     * @see #setDelayCountdown(int)
     */
	public int getDelay() {
		return delay;
	}



    /**
     * The <code>delay</code> is the number of times {@link #updateAnimation()}
     * needs to be called before the <code>currentFrameIndex</code> changes. In
     * effect this controls the speed of the animation. <br>
     * @param delay the new frame delay
     * @see #getDelayCountdown()
     * @see #setDelayCountdown(int)
     * @throws IllegalArgumentException if delay is less than zero
     */
	public void setDelay(int delay) throws IllegalArgumentException {
		ParameterChecker.checkMin(delay, "delay", 0);
		this.delay = delay;
	}



    /**
     * Returns the number of times {@link #updateAnimation()} has to be called
     * before the current frame is changed to the next in sequence. If
     * <code>getCount() == 0</code> the current frame will change on the
     * next call to <code>updateAnimation()</code> and <code>count</code> will
     * reset to <code>delay</code>. <br>
     * @return the countdown to a frame change
     * @see #getDelay()
     */
	public int getDelayCountdown() {
		return delayCountdown;
	}



	/**
     * Sets the number of times {@link #updateAnimation()} has to be called
     * before the current frame is changed to the next in sequence. <br>
     * @throws IllegalArgumentException if count is less than zero
     */
	public void setDelayCountdown(int count) throws IllegalArgumentException {
		ParameterChecker.checkMin(count, "count", 0);
		this.delayCountdown = count;
	}



	/**
	 * Returns the current frame index.
	 */
	public int getCurrentFrameIndex() {
		return currentFrameIndex;
	}



	/**
	 * Sets the current frame index.
	 */
	public void setCurrentFrameIndex(int currentFrameIndex) {
		ParameterChecker.checkMinMax(currentFrameIndex, "currentFrameIndex", 0, frames.length - 1);
		this.currentFrameIndex = currentFrameIndex;
	}



	/**
	 * Returns the current frame (image index). If this anim block has no frames
	 * then 0 is returned. <br>
	 * @return the current image index
	 */
	public int getCurrentFrame() {
		if ((currentFrameIndex < 0) || (currentFrameIndex > (frames.length-1)))
			 return 0;
		return frames[currentFrameIndex];
	}



	/**
	 * Returns an array of frames (image indexs) that represent the animation sequence.
	 */
	public int[] getFrames() {
		return frames;
	}



	/**
	 * Sets the frames (image indexs) that represent the animation sequence.
	 * @throws IllegalArgumentException is <code>frames</code> is <code>null</code>
	 */
	public void setFrames(int[] frames) throws IllegalArgumentException {
		ParameterChecker.checkNotNull(frames, "frames");
		this.frames = frames;
	}




	/**
	 * Returns any user defined data.
	 */
	public int getUserData() {
		return userData;
	}



	/**
	 * Sets any user defined data.
	 */
	public void setUserData(int userData) {
		this.userData = userData;
	}


	/**
	 * Counts down the delay and updates the current frame to the next in the
	 * animation sequence.
	 */
	public void updateAnimation() {
		delayCountdown--;
		if (delayCountdown < 0) {
			delayCountdown = delay;

			switch (type) {
				case AN_NONE:
					break;

				case AN_LOOPF:
					currentFrameIndex++;
					if (currentFrameIndex >= frames.length)
						currentFrameIndex = 0;
					break;

				case AN_LOOPR:
					currentFrameIndex--;
					if (currentFrameIndex < 0)
						currentFrameIndex = frames.length - 1;
					break;

				case AN_ONCE:
					currentFrameIndex++;
					if (currentFrameIndex >= frames.length) {
						currentFrameIndex = 0;
						type = AN_ONCES;
					}
					break;

				case AN_ONCES:
					break;

				case AN_ONCEH:
					if (currentFrameIndex < (frames.length - 1))
						currentFrameIndex++;
					break;

				case AN_PPFF:
					currentFrameIndex++;
					if (currentFrameIndex >= frames.length) {
						currentFrameIndex -= 2;
						type = AN_PPFR;
					}
					break;

				case AN_PPFR:
					currentFrameIndex--;
					if (currentFrameIndex < 0) {
						currentFrameIndex += 2;
						type = AN_PPFF;
					}
					break;

				case AN_PPRF:
					currentFrameIndex--;
					if (currentFrameIndex < 0) {
						currentFrameIndex += 2;
						type = AN_PPRR;
					}
					break;

				case AN_PPRR:
					currentFrameIndex++;
					if (currentFrameIndex >= frames.length) {
						currentFrameIndex -= 2;
						type = AN_PPRF;
					}
					break;

				default:
					break;
			}
		}
	}

}
