package com.alienfactory.javamappy;

import com.alienfactory.javamappy.util.ParameterChecker;



// -----------------------------------------------------------------------------
/**
 * A simple bean class which represents a <code>Block</code> as used by
 * <code>Mappy</code>. <br>
 * <br>
 * A <code>Block</code> may contain up to 4 layers of images (1 background layer
 * and 3 foreground layers), 7 fields of User data, quadratic collision
 * information and 4 other flags. <br>
 *
 * @author  Steve Eynon
 **/
// -----------------------------------------------------------------------------
public class Block {

	/**
	 * In conjunction with <code>TOP</code> or <code>BOTTOM</code> this
	 * specifies a corner of the <code>Block</code>. Used when getting / setting
	 * a collision flag. <br>
	 * <br>
	 * Has a constant value of <code>4</code>.
	 * @see		#setCollisionFlag(int, boolean)
	 * @see		#getCollisionFlag(int)
	 **/
    public static final int 		LEFT 				= 4;

	/**
	 * In conjunction with <code>TOP</code> or <code>BOTTOM</code> this
	 * specifies a corner of the <code>Block</code>. Used when getting / setting
	 * a collision flag. <br>
	 * <br>
	 * Has a constant value of <code>8</code>.
	 * @see		#setCollisionFlag(int, boolean)
	 * @see		#getCollisionFlag(int)
	 **/
    public static final int 		RIGHT 				= 8;

	/**
	 * In conjunction with <code>LEFT</code> or <code>RIGHT</code> this
	 * specifies a corner of the <code>Block</code>. Used when getting / setting
	 * a collision flag. <br>
	 * <br>
	 * Has a constant value of <code>16</code>.
	 * @see		#setCollisionFlag(int, boolean)
	 * @see		#getCollisionFlag(int)
	 **/
    public static final int 		TOP 				= 16;

	/**
	 * In conjunction with <code>LEFT</code> or <code>RIGHT</code> this
	 * specifies a corner of the <code>Block</code>. Used when getting / setting
	 * a collision flag. <br>
	 * <br>
	 * Has a constant value of <code>32</code>.
	 * @see		#setCollisionFlag(int, boolean)
	 * @see		#getCollisionFlag(int)
	 **/
    public static final int 		BOTTOM 				= 32;

	/**
	 * A <code>Block</code> image layer used to get / set its image index. <br>
	 * <br>
	 * Has a constant value of <code>0</code>.
	 * @see		#setImageIndex(int, int)
	 * @see		#getImageIndex(int)
	 **/
	public static final int			BACKGROUND			= 0;

	/**
	 * A <code>Block</code> image layer used to get / set its image index. <br>
	 * <br>
	 * Has a constant value of <code>1</code>.
	 * @see		#setImageIndex(int, int)
	 * @see		#getImageIndex(int)
	 **/
	public static final int			FOREGROUND1			= 1;

	/**
	 * A <code>Block</code> image layer used to get / set its image index. <br>
	 * <br>
	 * Has a constant value of <code>2</code>.
	 * @see		#setImageIndex(int, int)
	 * @see		#getImageIndex(int)
	 **/
	public static final int			FOREGROUND2			= 2;

	/**
	 * A <code>Block</code> image layer used to get / set its image index. <br>
	 * <br>
	 * Has a constant value of <code>3</code>.
	 * @see		#setImageIndex(int, int)
	 * @see		#getImageIndex(int)
	 **/
	public static final int			FOREGROUND3			= 3;

	/**
	 * A minimum index for User Data. <br>
	 * <br>
	 * Has a constant value of <code>1</code>.
	 * @see		#setUserData(int, int)
	 * @see		#getUserData(int)
	 **/
	public static final int			MIN_USER_DATA		= 1;

	/**
	 * A minimum index for User Data. <br>
	 * <br>
	 * Has a constant value of <code>7</code>.
	 * @see		#setUserData(int, int)
	 * @see		#getUserData(int)
	 **/
	public static final int			MAX_USER_DATA		= 7;

	/**
	 * A User defined flag. <br>
	 * <br>
	 * Has a constant value of <code>0</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int			TRIGGER				= 0;

	/**
	 * A User defined flag. <br>
	 * <br>
	 * Has a constant value of <code>1</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int			UNUSED1				= 1;

	/**
	 * A User defined flag. <br>
	 * <br>
	 * Has a constant value of <code>2</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int			UNUSED2				= 2;

	/**
	 * A User defined flag. <br>
	 * <br>
	 * Has a constant value of <code>3</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int			UNUSED3				= 3;

	/**
	 * Used in Pillar Riser Mode to flag that the next <code>Block</code>
	 * should continue the pillar. <br>
	 * <br>
	 * Has a constant value of <code>1</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int			ATTACH_NEXT			= 1;

	/**
	 * Used in Pillar Riser Mode to flag that only the left hand side of the
	 * <code>Block</code> should be rendered. <br>
	 * <br>
	 * Has a constant value of <code>2</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int			LEFT_SIDE_PILLAR	= 2;

	/**
	 * Used in Pillar Riser Mode to flag that only the right hand side of the
	 * <code>Block</code> should be rendered. <br>
	 * <br>
	 * Has a constant value of <code>3</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int			RIGHT_SIDE_PILLAR	= 3;



	private 		int 			imageIndex[]		= new int[4];
	private 		int 			userData[]			= new int[7];
	private			boolean			collisionFlags[]	= new boolean[4];
	private			boolean			flags[]				= new boolean[4];
	private			String			textString;



	// -------------------------------------------------------------------------
	// Public & Private Methods (Hmm, I'm tempted to make the setters private!)
	// -------------------------------------------------------------------------

	/**
	 * Sets the image index of the given <code>Block</code> layer. <br>
	 * @param	imageLayer should be one of {@link #BACKGROUND},
	 * 			{@link #FOREGROUND1}, {@link #FOREGROUND2} or
	 * 			{@link #FOREGROUND3}
	 * @param	newImageIndex the image index to be set
	 * @throws	IllegalArgumentException if <code>imageLayer</code> is out of
	 * 			bounds - it is assumed that <code>newImageIndex</code> is valid
	 * @see		#getImageIndex(int)
	 **/
	public void setImageIndex(int imageLayer, int newImageIndex) throws IllegalArgumentException  {
		ParameterChecker.checkMinMax(imageLayer, "imageLayer", BACKGROUND, FOREGROUND3);
		imageIndex[imageLayer] = newImageIndex;
	}



	/**
	 * Returns the image index of the given <code>Block</code> layer. <br>
	 * @param	imageLayer should be one of {@link #BACKGROUND},
	 * 			{@link #FOREGROUND1}, {@link #FOREGROUND2} or
	 * 			{@link #FOREGROUND3}
	 * @return	the image index of the given <code>Block</code> layer.
	 * @throws	IllegalArgumentException if <code>imageLayer</code> is out of
	 * 			bounds
	 * @see		#setImageIndex(int, int)
	 **/
	public int getImageIndex(int imageLayer) throws IllegalArgumentException {
		if ((imageLayer < BACKGROUND) || (imageLayer > FOREGROUND3))
			 throw new IllegalArgumentException("Input parameter [imageLayer :: " + imageLayer + "] was out of bounds :: " + BACKGROUND + " >= imageLayer =< " + FOREGROUND3);

		// return image index
		return (imageIndex[imageLayer]);
	}




	/**
	 * Sets the specified User Data. <br>
	 * @param	index the User Data to be updated.
	 * @param	newUserData	the new User Data.
	 * @throws	IllegalArgumentException if
	 * 			(<code>index</code> &lt; {@link #MIN_USER_DATA}) or
	 * 			(<code>index</code> &gt; {@link #MAX_USER_DATA}) or
	 * @see		#getUserData(int)
	 **/
	public void setUserData(int index, int newUserData) throws IllegalArgumentException {
		ParameterChecker.checkMinMax(index, "index", MIN_USER_DATA, MAX_USER_DATA);
		userData[index - 1] = newUserData;
	}



	/**
	 * Returns the specified User Data. <br>
	 * @param	index the User Data to be returned.
	 * @return	the specified User Data.
	 * @throws	IllegalArgumentException if
	 * 			(<code>index</code> &lt; {@link #MIN_USER_DATA}) or
	 * 			(<code>index</code> &gt; {@link #MAX_USER_DATA}) or
	 * @see		#setUserData(int, int)
	 **/
	public int getUserData(int index) throws IllegalArgumentException {
		if ((index < MIN_USER_DATA) || (index > MAX_USER_DATA))
			 throw new IllegalArgumentException("Input parameter [index :: " + index + "] was out of bounds :: " + MIN_USER_DATA + " >= index =< " + MAX_USER_DATA);

		// return User Data
		return userData[index - 1];
	}



	/**
	 * Sets the specified flag to the given boolean value. <br>
	 * <br>
	 * @param	flag should one of {@link #TRIGGER}, {@link #UNUSED1},
	 * 			{@link #UNUSED2} or {@link #UNUSED3}
	 * @param	newFlag	the new flag to set
	 * @throws	IllegalArgumentException if <code>flag</code> is out of bounds
	 * @see		#getFlag(int)
	 **/
	public void setFlag(int flag, boolean newFlag) throws IllegalArgumentException {
		ParameterChecker.checkMinMax(flag, "flag", TRIGGER, UNUSED3);
		flags[flag] = newFlag;
	}



	/**
	 * Returns the specified flag. <br>
	 * <br>
	 * @param	flag should be one of {@link #TRIGGER}, {@link #UNUSED1},
	 * 			{@link #UNUSED2} or {@link #UNUSED3}
	 * @return	the specified flag value
	 * @throws	IllegalArgumentException if <code>flag</code> is out of bounds
	 * @see		#setFlag(int, boolean)
	 **/
	public boolean getFlag(int flag) throws IllegalArgumentException {
		if ((flag < TRIGGER) || (flag > UNUSED3))
			throw new IllegalArgumentException("Input parameter [flag :: " + flag + "] was out of bounds :: " + TRIGGER + " >= flag =< " + UNUSED3);

		// return Flag Data
		return flags[flag];
	}



	/**
	 * Sets the specified collision flag. Example usage: <br>
	 * <blockquote><code>
	 * 		block.setCollisionFlag(TOP | LEFT, true);
	 * </code></blockquote>
	 * @param	flag should be a combination of {@link #LEFT}, {@link #RIGHT},
	 * 			{@link #TOP} and {@link #BOTTOM}
	 * @param	newFlag	the new Flag.
	 * @throws	IllegalArgumentException when <code>flag</code> is not valid
	 * @see		#getCollisionFlag(int)
	 **/
	public void setCollisionFlag(int flag, boolean newFlag) throws IllegalArgumentException {
		int index = -1;

		if (flag == (TOP | LEFT))		index = 0;
		if (flag == (TOP | RIGHT))		index = 1;
		if (flag == (BOTTOM | LEFT))	index = 2;
		if (flag == (BOTTOM | RIGHT))	index = 3;

		if (index == -1)
			 throw new IllegalArgumentException("Input parameter [flag] was out of bounds :: flag=[" + flag + "]");

		// set new Flag Data
		collisionFlags[index] = newFlag;
	}



	/**
	 * Returns the specified collision flag. Example usage: <br>
	 * <blockquote><code>
	 * 		block.getCollisionFlag(BOTTOM | RIGHT);
	 * </code></blockquote>
	 * @param	flag should be a combination of {@link #LEFT}, {@link #RIGHT},
	 * 			{@link #TOP} and {@link #BOTTOM}
	 * @return	the specified collision flag
	 * @throws	IllegalArgumentException when <code>flag</code> is not valid
	 * @see		#setCollisionFlag(int, boolean)
	 **/
	public boolean getCollisionFlag(int flag) throws IllegalArgumentException {
		int index = -1;
		if (flag == (TOP | LEFT))		index = 0;
		if (flag == (TOP | RIGHT))		index = 1;
		if (flag == (BOTTOM | LEFT))	index = 2;
		if (flag == (BOTTOM | RIGHT))	index = 3;

		if (index == -1)
			 throw new IllegalArgumentException("Input parameter [flag] was out of bounds :: flag=[" + flag + "]");

		// return Flag Data
		return collisionFlags[index];
	}


	/**
	 * Returns the text string associated with this block. The string is set
	 * when the map is loaded and is defined within Mappy. <br>
	 */
	public String getTextString() {
		return textString;
	}


	/**
	 * Sets the text string associated with this block. <br>
	 */
	public void setTextString(String textString) {
		this.textString = textString;
	}
}
