package com.alienfactory.javamappy;



// -----------------------------------------------------------------------------
/**
 * MapHeader :: This class represents the data held within the MPHD chunk. <br>
 *
 * @author Steve Eynon
 */
// -----------------------------------------------------------------------------
public class MapHeader {

	private int mapVersionMajor;
	private int mapVersionMinor;

	private boolean lsb;

	private int mapType;

	private int mapWidth;
	private int mapHeight;
//	private int reserved1;		// not used
//	private int reserved2;		// not used
	private int blockWidth;
	private int blockHeight;
	private int colourDepth;
	private int blockSize;
	private int noOfBlocks;
	private int noOfImages;

	private int	chromeKey;
	private int	chromeKeyIndex;

	private int blockGapX;
	private int blockGapY;
	private int blockStaggerX;
	private int blockStaggerY;
	private int clickMask;
	private boolean risingPillarMode;

	// this is defined in the TSTR chunk, but here seemed like a good plave to
	// stick it for now
	private int textStringUserDataIndex	= -1;



	public boolean isLsb() {
		return lsb;
	}

	public int getBlockGapX() {
		return blockGapX;
	}

	public int getBlockGapY() {
		return blockGapY;
	}

	/**
	 * Returns the height of a {@link Block} in pixels. <br>
	 */
	public int getBlockHeight() {
		return blockHeight;
	}

	public int getBlockSize() {
		return blockSize;
	}

	public int getBlockStaggerX() {
		return blockStaggerX;
	}

	public int getBlockStaggerY() {
		return blockStaggerY;
	}

	/**
	 * Returns the width of a {@link Block} in pixels. <br>
	 */
	public int getBlockWidth() {
		return blockWidth;
	}

	/**
	 * For colour depths 16 and 24 this returns the Colour that is
	 * rendered transparent. The colour is in 0xAARRGGBB format where an alpha
	 * of 0xFF is fully opaque.<br>
	 */
	public int getChromeKey() {
		return chromeKey;
	}

	/**
	 * For a colour depths of 8, this returns the colour index into
	 * {@link Map#getColours()} that is rendered transparent. <br>
	 * rendered transparent. <br>
	 */
	public int getChromeKeyIndex() {
		return chromeKeyIndex;
	}

	public int getClickMask() {
		return clickMask;
	}

	/**
	 * Returns the colour depth of the block images. Should be one of either 8,
	 * 16, 24 or 32. <br>
	 */
	public int getColourDepth() {
		return colourDepth;
	}

	/**
	 * Returns the height of the {@link Map} in <code>Blocks</code>. <br>
	 */
	public int getMapBlockHeight() {
		return mapHeight;
	}

	/**
	 * Returns the height of the {@link Map} in pixels. This takes into account
	 * the staggered block layout of an isometric map. <br>
	 */
	public int getMapPixelHeight() {
		int height;
		if (blockStaggerY != 0)
			height = (blockStaggerY * mapHeight) - blockStaggerY;
		else
			height = mapHeight * blockHeight;
		return height;
	}

	/**
	 * I'm not really sure what this is, but from my own observations I've
	 * noticed the following: <br>
	 * <ul>
	 * 		<li>0 = FMP 0.5</li>
	 * 		<li>1 = FMP 1.0</li>
	 * 		<li>2 = FMP 1.0 RLE (Run Length Encoding)</li>
	 * </ul>
	 */
	public int getMapType() {
		return mapType;
	}

	/**
	 * Returns the major part of the FMP version. <br>
	 */
	public int getMapVersionMajor() {
		return mapVersionMajor;
	}

	/**
	 * Returns the minor part of the FMP version. <br>
	 */
	public int getMapVersionMinor() {
		return mapVersionMinor;
	}

	/**
	 * Returns the width of the {@link Map} in <code>Blocks</code>. <br>
	 */
	public int getMapBlockWidth() {
		return mapWidth;
	}

	/**
	 * Returns the width of the {@link Map} in pixels. This takes into account
	 * the staggered block layout of an isometric map. <br>
	 */
	public int getMapPixelWidth() {
		int width;
		if (blockStaggerX != 0)
			width = (blockStaggerX * 2 * mapWidth) - blockStaggerX;
		else
			width = mapWidth * blockWidth;
		return width;
	}

	public int getNoOfBlocks() {
		return noOfBlocks;
	}

	public int getNoOfImages() {
		return noOfImages;
	}

	/**
	 * Rising Pillar Mode is essentially Isometric Mode. <br>
	 */
	public boolean isRisingPillarMode() {
		return risingPillarMode;
	}

	/**
	 * Used by the Block Chunk Loader. A value of -1 means text strings have not
	 * been set. <br>
	 */
	public int getTextStringUserDataIndex() {
		return textStringUserDataIndex;
	}



	public void setLsb(boolean lsb) {
		this.lsb = lsb;
	}
	public void setBlockGapX(int blockGapX) {
		this.blockGapX = blockGapX;
	}
	public void setBlockGapY(int blockGapY) {
		this.blockGapY = blockGapY;
	}
	public void setBlockHeight(int blockHeight) {
		this.blockHeight = blockHeight;
	}
	public void setBlockSize(int blockSize) {
		this.blockSize = blockSize;
	}
	public void setBlockStaggerX(int blockStaggerX) {
		this.blockStaggerX = blockStaggerX;
	}
	public void setBlockStaggerY(int blockStaggerY) {
		this.blockStaggerY = blockStaggerY;
	}
	public void setBlockWidth(int blockWidth) {
		this.blockWidth = blockWidth;
	}
	public void setChromeKey(int chromeKey) {
		this.chromeKey = chromeKey;
	}
	public void setChromeKeyIndex(int chromeKeyIndex) {
		this.chromeKeyIndex = chromeKeyIndex;
	}
	public void setClickMask(int clickMask) {
		this.clickMask = clickMask;
	}
	public void setColourDepth(int colourDepth) {
		this.colourDepth = colourDepth;
	}
	public void setMapBlockHeight(int mapHeight) {
		this.mapHeight = mapHeight;
	}
	public void setMapType(int mapType) {
		this.mapType = mapType;
	}
	public void setMapVersionMajor(int mapVersionMajor) {
		this.mapVersionMajor = mapVersionMajor;
	}
	public void setMapVersionMinor(int mapVersionMinor) {
		this.mapVersionMinor = mapVersionMinor;
	}
	public void setMapBlockWidth(int mapWidth) {
		this.mapWidth = mapWidth;
	}
	public void setNoOfBlocks(int noOfBlocks) {
		this.noOfBlocks = noOfBlocks;
	}
	public void setNoOfImages(int noOfImages) {
		this.noOfImages = noOfImages;
	}
	public void setRisingPillarMode(boolean risingPillarMode) {
		this.risingPillarMode = risingPillarMode;
	}
	public void setTextStringUserDataIndex(int textStringUserDataIndex) {
		this.textStringUserDataIndex = textStringUserDataIndex;
	}
}
