package com.alienfactory.javamappy.loader;

import com.alienfactory.javamappy.Map;



// ----------------------------------------------------------------------------
/**
 * This is the base class for all ChunkLoaders. <br>
 *
 * @author Steve Eynon
 **/
// ----------------------------------------------------------------------------
public abstract class ChunkLoader {

	private String			chunkName		= "";
    private byte[]          rawChunkData 	= new byte[0];
    private int             rawChunkIndex;
	private static boolean	lsb;



    //--------------------------------------------------------------------------
    // Public Methods
    //--------------------------------------------------------------------------

	/**
	 * Returns the total number of bytes in this chunk. <br>
	 */
	public int getChunkLength() {
		return rawChunkData.length;
	}



	/**
	 * Returns the 4 character name of the chunk. E.g. ANDT, BODY. <br>
	 */
	public String getChunkName() {
		return chunkName;
	}



	/**
	 * Returns the number of <code>byte</code>s yet to be read for this chunk. <br>
	 * <br>
	 * @return the number of bytes left to read
	 **/
	public int getBytesRemaining() {
		return (rawChunkData.length - rawChunkIndex);
	}



	/**
	 * Pretty print. <br>
	 */
	public String toString() {
		StringBuffer buff = new StringBuffer();

		buff.append("ChunkLoader");
		buff.append(" :: ");
		buff.append("Name=[" + getChunkName() + "]");
		buff.append(", ");
		buff.append("Length=[" + getChunkLength() + "]");

		return buff.toString();
	}



	/**
	 * The method invoked to load the chunk. All data loaded should by placed in
	 * the given Map. <br>
	 * <br>
	 * @throws ChunkLoadException should be thrown should anything be wrong with
	 * the data you are loading
	 **/
	public abstract void loadChunk(Map map) throws ChunkLoadException;



    //--------------------------------------------------------------------------
    // Protected Methods
    //--------------------------------------------------------------------------

	/**
	 * Returns true if shorts and int are stored Least Significant Byte first in
	 * the inputStream. I advise using the {@link #readShort()} or
	 * {@link #readInt()} methods as these take care of LSB effects for you.
	 */
	protected boolean isLSB() {
		return lsb;
	}



	/**
	 * Returns the byte at the given index without affecting the 'inputStream'.
	 */
	protected byte getByte(int index) {
		return rawChunkData[index];
	}



	/**
	 * Throws a <code>ChunkLoadException</code> with a formatted message. <br>
	 * <br>
	 * @param	objMessage 		the error message
	 * @throws 	ChunkLoadException 	always.
	 **/
	protected void throwChunkException(Object objMessage) throws ChunkLoadException {
		throw new ChunkLoadException("Chunk " + getChunkName() + "> <" + objMessage.toString());
	}



	/**
	 * Reads the next <i>n</i> bytes from the Map input stream and turns them
	 * into a <code>String</code>. <br>
	 * <br>
	 * @param noOfChars the number of bytes to read and turn into a <code>String</code>
	 * @return a <code>String</code>, read from the Map input stream
	 * @throws IllegalArgumentException if <code>intNoOfChars</code> is less than zero
	 * @throws ChunkLoadException if there are no more bytes left to read in the Chunk
	 **/
	protected String readString(int noOfChars) throws IllegalArgumentException, ChunkLoadException {
		if (noOfChars < 0) throw new IllegalArgumentException("Input parameter [intNoOfChars] can not be less than zero :: intNoOfChars=[" + noOfChars + "]");
		if (noOfChars > getBytesRemaining()) throwChunkException("No more bytes left in Chunk!");

		// convert the bytes into a String
        String strText = new String(rawChunkData, rawChunkIndex, noOfChars);
		rawChunkIndex += noOfChars;

		// return the string
        return strText;
	}



	/**
	 * Reads characters until it find a terminating zero and returns the string.
	 * Note the zero is consumed. <br>
	 * @return a <code>String</code>, read from the Map input stream
	 */
	protected String readString()  {
		StringBuffer stbChar = new StringBuffer();
		byte		 bytChar;
		do {
			bytChar = readByte();
			if (bytChar != 0)
				stbChar.append((char) bytChar);
		} while (bytChar != 0);

		return stbChar.toString();
	}



	/**
	 * Reads the next <code>byte</code> from the input stream. <br>
	 * <br>
	 * @return the next <code>byte</code> (signed) from the Map input stream
	 * @throws ChunkLoadException if there are no more bytes left to read in the Chunk
	 **/
	protected byte readByte() throws ChunkLoadException {
		if (1 > getBytesRemaining()) throwChunkException("No more bytes left in Chunk!");

		// return a byte from the array
		return rawChunkData[rawChunkIndex++];
	}



	/**
	 * Reads the next <code>byte</code> from the input stream and returns it as
	 * an unsigned <code>int</code>. <br>
	 * <br>
	 * @return the next <code>byte</code> (unsigned) from the Map input stream
	 * @throws ChunkLoadException if there are no more bytes left to read in the Chunk
	 **/
	protected int readUByte() throws ChunkLoadException {
		return (readByte() & 0xFF);
	}



	/**
	 * Reads the next 2 bytes from the Map input stream and turns them into a
	 * <code>short</code>. <br>
	 * <br>
	 * @return the next <code>short</code> from the Map input stream
	 * @throws ChunkLoadException if there are no more bytes left to read in the Chunk
	 **/
	protected short readShort() throws ChunkLoadException {
		if (2 > getBytesRemaining()) throwChunkException("No more bytes left in Chunk!");

        // default return value
        int value = 0;
        int index = rawChunkIndex;

        // take into account the effects of LSB
        if (lsb) {
            byte temp;
            temp					= rawChunkData[index];
            rawChunkData[index + 0] = rawChunkData[index + 1];
            rawChunkData[index + 1] = temp;
        }

        // loop for all the required bytes
        while (rawChunkIndex < (index + 2)) {
            value <<= 8;
            value |= rawChunkData[rawChunkIndex++] & 0xFF;
        }

		// and return the short
		return (short) value;
	}



	/**
	 * Reads the next 4 bytes from the Map input stream and turns them into an
	 * <code>int</code>. <br>
	 * <br>
	 * @return the next <code>int</code> from the Map input stream
	 * @throws ChunkLoadException if there are no more bytes left to read in the Chunk
	 **/
	protected int readInt() throws ChunkLoadException {
		if (4 > getBytesRemaining()) throwChunkException("No more bytes left in Chunk!");

        // default return value
        int value = 0;
        int index = rawChunkIndex;

        // take into account the effects of LSB
        if (lsb) {
            byte temp;
            temp					= rawChunkData[index];
            rawChunkData[index + 0] = rawChunkData[index + 3];
            rawChunkData[index + 3] = temp;
            temp					= rawChunkData[index + 1];
            rawChunkData[index + 1] = rawChunkData[index + 2];
            rawChunkData[index + 2] = temp;
        }

        // loop for all the required bytes
        while (rawChunkIndex < (index + 4)) {
            value <<= 8;
            value |= rawChunkData[rawChunkIndex++] & 0xFF;
        }

        // return int value
        return value;
	}



	/**
	 * Skips the next <i>n</i> <code>byte</code>s from the Map input stream. <br>
	 * <br>
	 * @param noOfBytes the number of <code>byte</code>s to skip
	 * @throws IllegalArgumentException	if <code>lngNoOfBytes</code> is less than zero
	 * @throws ChunkLoadException if there are no more bytes left to read in the Chunk
	 **/
	protected void skip(int noOfBytes) throws IllegalArgumentException, ChunkLoadException {
		if (noOfBytes < 0) throw new IllegalArgumentException("Input parameter [noOfBytes] can not be less than zero :: lngNoOfBytes=[" + noOfBytes + "]");
		if (noOfBytes > getBytesRemaining()) throwChunkException("No more bytes left in Chunk!");

		rawChunkIndex += noOfBytes;
	}



    //--------------------------------------------------------------------------
    // Package Methods
    //--------------------------------------------------------------------------

    /**
     * Called by <code>MapLoader.createChunkLoader()</code> to initialise the
     * <code>ChunkLoader</code>.  This method gets us out of having complex
     * constructors and means I can change it at any time without impacting on
     * any existing code.
     **/
    void initialise(String chunkyName, byte[] rawChunkyData) throws IllegalArgumentException {
        if (chunkName == null)		throw new IllegalArgumentException("Input parameter [chunkyName] was null.");
        if (rawChunkyData == null)	throw new IllegalArgumentException("Input parameter [rawChunkyData] was null.");

        // set member variables
        rawChunkIndex	= 0;
        chunkName		= chunkyName;
        rawChunkData	= rawChunkyData;
    }



	/**
	 * This should only ever be called by the Chunk Loader for MPHD. It affects
	 * how all the map data is loaded. <br>
	 * <br>
	 * @param blnLSB if <code>true</code> then the Map data is read in LSB format, otherwise MSB is used
	 **/
	void setLSB(boolean blnLSB) {
		lsb = blnLSB;
	}
}
