package com.alienfactory.javamappy.loader;

import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.MapHeader;
import com.alienfactory.javamappy.util.Logger;



// -----------------------------------------------------------------------------
/**
 * Chunk MPHD :: Map header, see struct in mapfunc.c<br>
 *
 * @author Steve Eynon
 **/
// -----------------------------------------------------------------------------
public class ChunkLoaderMPHD extends ChunkLoader {
	private final Logger log = Logger.createLogger(super.getClass());



	// -------------------------------------------------------------------------
	// Public Methods
	// -------------------------------------------------------------------------

	// JavaDoc inherited from ChunkLoader
	public void loadChunk(Map map) throws ChunkLoadException {

		// read and verify the map version
		int mapVersionMajor = readByte();
		int mapVersionMinor = readByte();
		if (mapVersionMajor > 1) throwChunkException("Map version is unsupported :: version=[" + mapVersionMajor + "." + mapVersionMinor + "]");

		// this has to be the most important bit in the whole damn map!
		boolean lsb = (readByte() == 1);
		super.setLSB(lsb);

		// read and verify the map type
		int mapType = readByte();
		if (mapType > 3) throwChunkException("Map type is unsupported :: type=[" + mapType + "]");

		// read general map data
		int mapWidth	= readShort();
		int mapHeight	= readShort();
		int reserved1	= readShort();
		int reserved2	= readShort();
		int blockWidth	= readShort();
		int blockHeight	= readShort();
		int colourDepth	= readShort();
		int blockSize	= readShort();
		int noOfBlocks	= readShort();
		int noOfImages	= readShort();

		// default values
		int cKey8Bit	= 0;
		int cKeyRed		= 0;
		int cKeyGreen	= 0;
		int cKeyBlue	= 0;

		// these bytes were added in FMP v0.4, but it's easier and safer to just
		// check the number of bytes left
		if (super.getBytesRemaining() >= 4) {
			cKey8Bit	= readUByte();
			cKeyRed		= readUByte();
			cKeyGreen	= readUByte();
			cKeyBlue	= readUByte();
		}

		// default values
		int blockGapX	= blockWidth;
		int blockGapY	= blockHeight;
		int blockStaggerX= 0;
		int blockStaggerY= 0;
		int clickMask	= -1;
		int pillars		= -1;

		// these bytes were added in FMP v0.5, but it's easier and safer to just
		// check the number of bytes left
		if (super.getBytesRemaining() >= 12) {
			blockGapX	= readShort();
			blockGapY	= readShort();
			blockStaggerX= readShort();
			blockStaggerY= readShort();
			clickMask	= readShort();
			pillars		= readShort();
		}

		// log the values
		log.debug("FMP Version     = [" + mapVersionMajor + "." + mapVersionMinor + "]");
		log.debug("LSB             = [" + lsb + "]");
		log.debug("Map Type        = [" + mapType + "]");
		log.debug("Map Width       = [" + mapWidth + "]");
		log.debug("Map Height      = [" + mapHeight + "]");
		log.debug("Reserved 1      = [" + reserved1 + "]");
		log.debug("Reserved 2      = [" + reserved2 + "]");
		log.debug("Block Width     = [" + blockWidth + "]");
		log.debug("Block Height    = [" + blockHeight + "]");
		log.debug("Colour Depth    = [" + colourDepth + "]");
		log.debug("Block Size      = [" + blockSize + "]");
		log.debug("No. Of Blocks   = [" + noOfBlocks + "]");
		log.debug("No. Of Images   = [" + noOfImages + "]");
		log.debug("CKey 8 Bit      = [" + cKey8Bit + "]");
		log.debug("CKey Red        = [" + cKeyRed + "]");
		log.debug("CKey Green      = [" + cKeyGreen + "]");
		log.debug("CKey Blue       = [" + cKeyBlue + "]");
		log.debug("Block Gap X     = [" + blockGapX + "]");
		log.debug("Block Gap Y     = [" + blockGapY + "]");
		log.debug("Block Stagger X = [" + blockStaggerX + "]");
		log.debug("Block Stagger Y = [" + blockStaggerY + "]");
		log.debug("Click Mask      = [" + clickMask + "]");
		log.debug("Pillars         = [" + pillars + "]");

		// copy the values into a MapHeader object
		MapHeader mapHeader 		= map.getMapHeader();
		mapHeader.setMapVersionMajor(mapVersionMajor);
		mapHeader.setMapVersionMinor(mapVersionMinor);
		mapHeader.setLsb			(lsb);
		mapHeader.setMapType		(mapType);
		mapHeader.setMapBlockWidth	(mapWidth);
		mapHeader.setMapBlockHeight	(mapHeight);
		mapHeader.setBlockWidth		(blockWidth);
		mapHeader.setBlockHeight	(blockHeight);
		mapHeader.setColourDepth	(colourDepth);
		mapHeader.setBlockSize		(blockSize);
		mapHeader.setNoOfBlocks		(noOfBlocks);
		mapHeader.setNoOfImages		(noOfImages);
		mapHeader.setChromeKeyIndex	(cKey8Bit);
		mapHeader.setChromeKey		((0xFF << 24) | (cKeyRed << 16) | (cKeyGreen << 8) | cKeyBlue);
		mapHeader.setBlockGapX		(blockGapX);
		mapHeader.setBlockGapY		(blockGapY);
		mapHeader.setBlockStaggerX	(blockStaggerX);
		mapHeader.setBlockStaggerY	(blockStaggerY);
		mapHeader.setClickMask		(clickMask);
		mapHeader.setRisingPillarMode(pillars > 0);
	}
}
