package com.alienfactory.javamappy.loader;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

import com.alienfactory.javamappy.JavaMappyException;
import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.util.Logger;
import com.alienfactory.javamappy.util.ParameterChecker;



// -----------------------------------------------------------------------------
/**
 * MapLoader <br>
 * @author Slimer
 */
// -----------------------------------------------------------------------------
public class MapLoader {
	private static final Logger log = Logger.createLogger("MapLoader");

	protected final Hashtable chunkLoaders = new Hashtable();



    // -------------------------------------------------------------------------
    // Public Constructor
    // -------------------------------------------------------------------------

	public MapLoader() {
		// by having instances of MapLoaders they become self initialising and
		// the ChunkLoader instances (below) get automatically garbage collected
		registerChunkLoaders();
	}



    // -------------------------------------------------------------------------
    // Public Helper Methods
    // -------------------------------------------------------------------------

	/**
	 * Helper method to create a {@link Map} from the given input stream of .FMP
	 * data. <br>
	 * <br>
	 * Note that this method does not close your input stream. <br>
	 */
	public static Map loadMap(InputStream inputStream) throws IllegalArgumentException, JavaMappyException {
		return new MapLoader().doLoadMap(inputStream);
	}



    // -------------------------------------------------------------------------
    // Protected Methods
    // -------------------------------------------------------------------------

	/**
	 * Performs the actual FMP map loading. Not static to allow overriding. <br>
	 */
	protected Map doLoadMap(InputStream inputStream) throws IllegalArgumentException, JavaMappyException {
		if (inputStream == null) throw new IllegalArgumentException("Input parameter [inputStream] was null");

		log.trace("loadMap() - Starting...");
		log.info("Loading Map from InputStream");

		Map 		map = new Map();
		ChunkLoader	chkMapHeader;
		long        lngBytesToRead;

		try {
			// load the map header
			chkMapHeader = createChunkLoader(inputStream);
			if (!chkMapHeader.getChunkName().equals("FORM"))
				throw new JavaMappyException("Map does not start with chunk [FORM] :: Chunk ID = [" + chkMapHeader.getChunkName() + "]");

			// find out how many bytes are contained in the map data
			chkMapHeader.loadChunk(null);
			lngBytesToRead	= chkMapHeader.getChunkLength() - 4;

			// read chunks while there's still chunks to read
			while (lngBytesToRead > 0) {
				ChunkLoader chunkLoader;

				// find and create the next chunk loader
				chunkLoader = createChunkLoader(inputStream);

				// load the chunk
				chunkLoader.loadChunk(map);

				// check that all the data was loaded
				if (chunkLoader.getBytesRemaining() > 0) {
					log.warn("Chunk [" + chunkLoader.getChunkName() + "] still has [" + chunkLoader.getBytesRemaining() + "] bytes remaining, skipping...");
				}

				// the countdown continues
				lngBytesToRead -= 8;	// for the chunk header
				lngBytesToRead -= chunkLoader.getChunkLength();
			}

		} catch (JavaMappyException jme) {
			// let map exceptions trickle through
			throw jme;

		} catch (Exception e) {
			throw new JavaMappyException("Could not load map", e);
		}

		log.trace("loadMap() - Done.");
		return map;

	}



	/**
     * Factory method for producing instances of ChunkLoaders. <br>
	 * Note: This method does not close the given input stream. <br>
	 *
	 * @return a <code>ChunkLoader</code> for the given <code>inputStream</code>
	 */
	protected ChunkLoader createChunkLoader(InputStream inputStream) throws IllegalArgumentException, ChunkLoadException {
		ParameterChecker.checkNotNull(inputStream, "inputStream");

		ChunkLoader chunkLoader 	= null;
		String		errorMessage	= null;

		log.trace("createChunkLoader() - Starting...");

		try {
			byte 	array[] 		= new byte[4];

			errorMessage			= "Could not read the chunk ID.";
			readStream(inputStream, array, 4);
			String	chunkID 		= new String(array);

			errorMessage			= "Could not read the chunk length for Chunk ID [" + chunkID + "]";
			readStream(inputStream, array, 4);
			int	intChunkLength		= 0;	// ignore the LSB / MSB thing for chunk lengths
			for (int i=0; i < 4; i++) {
				intChunkLength <<= 8;
				intChunkLength |= (array[i] & 0xFF);
			}

            errorMessage         = "Could not read the raw chunk data for Chunk ID [" + chunkID + "]";
            array                = new byte[intChunkLength];
            if (chunkID.equals("FORM"))  // just a little fudge!
            	readStream(inputStream, array, 4);
            else
            	readStream(inputStream, array, intChunkLength);

			// set a default return value
			errorMessage			= "Problem setting Default Chunk Loader attributes.";
			chunkLoader				= new DefaultChunkLoader();
            chunkLoader.initialise(chunkID, array);

			errorMessage			= "Could not find a ChunkLoader for ID=[" + chunkID + "].";
			ChunkLoader newChunkL	= (ChunkLoader) chunkLoaders.get(chunkID);
			if (newChunkL == null)	throw new RuntimeException();
			chunkLoader				= newChunkL;

			errorMessage        	= "Problem initialiising Chunk Loader.";
            chunkLoader.initialise(chunkID, array);

		} catch (Exception e) {
			// always return a default chunk loader if we can, so we may
			// continue reading the rest of the map. Not finding a particular
			// ChunkLoader is going to be a common problem.
			if (chunkLoader != null) {
				log.warn(errorMessage);
			} else {
				throw new ChunkLoadException(errorMessage, e);
			}
		}

		log.info(chunkLoader);
		log.trace("createChunkLoader() - Done.");
		return chunkLoader;
	}



	/**
	 * This is where we hard code chunk loader class references to chunk names.
	 * We do this so that (J2ME) obfuscaters can rename classes. It also allows
	 * us to delete empty chunk loader classes like ChunkLoaderLRYx. <br>
	 */
	protected void registerChunkLoaders() {

		// we have to create instances of the chunkloaders 'cos J2ME doesn't
		// have the concept of "ChunkLoaderXXXX.class". Annoying huh?
		registerChunkLoader("ANDT", new ChunkLoaderANDT());
		registerChunkLoader("ATHR", new ChunkLoaderATHR());
		registerChunkLoader("BGFX", new ChunkLoaderBGFX());
		registerChunkLoader("BKDT", new ChunkLoaderBKDT());
		registerChunkLoader("CMAP", new ChunkLoaderCMAP());
		registerChunkLoader("FORM", new ChunkLoaderFORM());
		registerChunkLoader("MPHD", new ChunkLoaderMPHD());
	    registerChunkLoader("OBDT", new ChunkLoaderOBDT());
	    registerChunkLoader("OBFN", new ChunkLoaderOBFN());
		registerChunkLoader("TSTR", new ChunkLoaderTSTR());

		ChunkLoaderBODY chunkLoaderBODY = new ChunkLoaderBODY();
		registerChunkLoader("BODY", chunkLoaderBODY);
		registerChunkLoader("LYR1", chunkLoaderBODY);
		registerChunkLoader("LYR2", chunkLoaderBODY);
		registerChunkLoader("LYR3", chunkLoaderBODY);
		registerChunkLoader("LYR4", chunkLoaderBODY);
		registerChunkLoader("LYR5", chunkLoaderBODY);
		registerChunkLoader("LYR6", chunkLoaderBODY);
		registerChunkLoader("LYR7", chunkLoaderBODY);
	}



	protected void registerChunkLoader(String name, ChunkLoader chunkLoader) {
		chunkLoaders.put(name, chunkLoader);
	}



    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------

    /**
     * This method replaces the standard InputStream.read(byte[], int, int).
     * Reason being, in Java1.1.x, it didn't friggin work! Believe me, I spent
     * hours on it! You keep getting misc and spurlious data creeping in - VERY
     * annoying, I can tell you! Grrr... <br>
     **/
    private static void readStream(InputStream stream, byte bytArray[], int size) throws IOException {
        int intIndex = 0;
        while ((intIndex > -1) && (intIndex < size)) {
            intIndex += stream.read(bytArray, intIndex, size - intIndex);
        }
    }



    // -------------------------------------------------------------------------
    // Protected Helper Classes
    // -------------------------------------------------------------------------

    protected static class DefaultChunkLoader extends ChunkLoader {
		public void loadChunk(Map map) {
			// prevent the warning log
			super.skip(super.getBytesRemaining());
		}
	}
}
