package com.alienfactory.javamappy.viewer;

import com.alienfactory.javamappy.AnimBlock;
import com.alienfactory.javamappy.Block;
import com.alienfactory.javamappy.Layer;
import com.alienfactory.javamappy.MapHeader;
import com.alienfactory.javamappy.util.Logger;
import com.alienfactory.javamappy.util.ParameterChecker;
import com.alienfactory.javamappy.viewer.render.Renderer;



// -----------------------------------------------------------------------------
/**
 * Renders a <code>Layer</code> of a <code>Map</code> onto the screen. This
 * class may be used independently (or even without) a <code>MapViewer</code>.
 * <br>
 * <br>
 * See {@link MapViewer} for a discussion on offsets. <br>
 *
 * @author Steve Eynon
 **/
// -----------------------------------------------------------------------------
public class LayerViewer extends Viewer {
	private final Logger log = Logger.createLogger(super.getClass());

	private final Layer	layer;
	private final int 	blockGapX,		blockGapY;
	private final int 	staggerX,		staggerY;
	private final int	tallestPillar;
	private final int	mapBlockHeight;



	// -------------------------------------------------------------------------
	// Constructors
	// -------------------------------------------------------------------------

	/**
	 * Creates a <code>LayerViewer</code> and places the viewing area at the
	 * top left corner. <br>
	 * @param layer			the <code>Layer</code> to be viewed
	 * @param mapHeader		contains useful information
	 * @param renderer		the <code>Renderer</code> to use when drawing the map
	 * @param viewWidth		the width of the viewing area in pixels
	 * @param viewHeight	the height of the viewing area in pixels
	 */
	public LayerViewer(Layer layer, MapHeader mapHeader, Renderer renderer, int viewWidth, int viewHeight) {
		this(layer, mapHeader, renderer, viewWidth, viewHeight, 0, 0);
	}



	/**
	 * Creates a <code>LayerViewer</code> and places the viewing area at the
	 * given offset. <br>
	 * @param layer			the <code>Layer</code> to be viewed
	 * @param mapHeader		contains useful information
	 * @param renderer		the <code>Renderer</code> to use when drawing the map
	 * @param viewWidth		the width of the viewing area in pixels
	 * @param viewHeight	the height of the viewing area in pixels
	 * @param viewOffsetX	the initial offset in pixels of the viewing area
	 * 		  				from the top left hand corner of the screen
	 * @param viewOffsetY	the initial offset in pixels of the viewing area
	 * 						from the top left hand corner of the screen
	 */
	public LayerViewer(Layer layer, MapHeader mapHeader, Renderer renderer, int viewWidth, int viewHeight, int viewOffsetX, int viewOffsetY) {
		super(mapHeader, renderer, viewWidth, viewHeight, viewOffsetX, viewOffsetY);
		ParameterChecker.checkNotNull(layer, "layer");

		this.layer		= layer;
		blockGapX		= mapHeader.getBlockGapX();
		blockGapY		= mapHeader.getBlockGapY();
		staggerX		= mapHeader.getBlockStaggerX();
		staggerY		= mapHeader.getBlockStaggerY();
		mapBlockHeight	= mapHeader.getMapBlockHeight();

		// grab the height of the tallest pillar
		tallestPillar = calculateTallestPillar();
	}



	// -------------------------------------------------------------------------
	// Public Methods
	// -------------------------------------------------------------------------

	/**
	 * Calculates the height of the tallest pillar - used when drawing
	 * Isometric maps with Pillar Riser Mode turned on. <br>
	 * @return	the height of the tallest pillar in pixels
	 */
	public int calculateTallestPillar() {
		int tallest = 0;
		int pillarHeight, pillarTop = 0, tower = 0;
		int blockHeight = mapHeader.getBlockHeight();
		Block block;
		Block[] blocks = layer.getBlocks();
		for (int i = 0; i < blocks.length; i++) {
			block = blocks[i];

			pillarHeight = tower = 0;
			if (block.getImageIndex(Block.BACKGROUND) != 0)
				pillarHeight = blockHeight * 1;
			if (block.getImageIndex(Block.FOREGROUND1) != 0)
				pillarHeight = blockHeight * 1;
			if (block.getImageIndex(Block.FOREGROUND2) != 0)
				pillarHeight = blockHeight * 2;
			if (block.getImageIndex(Block.FOREGROUND3) != 0)
				pillarHeight = blockHeight * 3;

			while (blocks[i].getFlag(Block.ATTACH_NEXT)) {
				// we don't like index out of bounds exceptions
				if (i++ >= blocks.length) break;
				block = blocks[i];
				pillarTop = 0;

				if (block.getImageIndex(Block.BACKGROUND) != 0)
					pillarTop = 1;
				if (block.getImageIndex(Block.FOREGROUND1) != 0)
					pillarTop = 2;
				if (block.getImageIndex(Block.FOREGROUND2) != 0)
					pillarTop = 3;
				if (block.getImageIndex(Block.FOREGROUND3) != 0)
					pillarTop = 4;

				tower += 4;
				if (pillarTop > 0)
					pillarHeight = (tower + pillarTop - 1) * blockHeight;
			}

			if (tallest < pillarHeight)
				tallest = pillarHeight;
		}
		log.debug("tallestPiller=[" + tallestPillar + "]");
		return tallest;
	}



	/**
	 * Renders the background layer of all the visible block onto the screen.
	 * <br>
	 * @param objGFX the gfx object specific to the <code>Renderer</code>
	 * @param transparency whether or not the layer should be rendered using
	 * their transparent pixels
	 */
	public void drawBackground(Object objGFX, boolean transparency) {
		draw(objGFX, transparency, true, false, false, false, 0, 0, super.viewWidth, super.viewHeight);
	}



	public void drawBackground(Object objGFX, boolean transparency, int x, int y, int width, int height) throws IllegalArgumentException{
		draw(objGFX, transparency, true, false, false, false, x, y, width, height);
	}



	/**
	 * Renders a foreground layer of all the visible blocks onto the screen.
	 * Transparency is always turned on when rendering foreground block layers.
	 * <br>
	 * @param objGfx the gfx object specific to the <code>Renderer</code>
	 * @param blockLayer the foreground layer to render
	 * @throws IllegalArgumentException if <code>blockLayer</code> is out of
	 * bounds or objGfx is <code>null</code>
	 */
	public void drawForeground(Object objGfx, int blockLayer) throws IllegalArgumentException {
		ParameterChecker.checkMinMax(blockLayer, "blockLayer", Block.FOREGROUND1, Block.FOREGROUND3);

		boolean drawForeground1 = (blockLayer == 1);
		boolean drawForeground2 = (blockLayer == 2);
		boolean drawForeground3 = (blockLayer == 3);

		draw(objGfx, true, false, drawForeground1, drawForeground2, drawForeground3, 0, 0, super.viewWidth, super.viewHeight);
	}



	public void drawForeground(Object objGfx, int blockLayer, int x, int y, int width, int height) throws IllegalArgumentException {
		ParameterChecker.checkMinMax(blockLayer, "blockLayer", Block.FOREGROUND1, Block.FOREGROUND3);

		boolean drawForeground1 = (blockLayer == 1);
		boolean drawForeground2 = (blockLayer == 2);
		boolean drawForeground3 = (blockLayer == 3);

		draw(objGfx, true, false, drawForeground1, drawForeground2, drawForeground3, x, y, width, height);
	}



	/**
	 * Renders the Background and Foreground layers of all the visible blocks
	 * onto the screen. <br>
	 * @param objGFX the gfx object specific to the <code>Renderer</code>
	 * @param transparency whether or not the background layer of the blocks
	 * should be rendered honouring their transparent pixels
	 */
	public void draw(Object objGFX, boolean transparency) {
		draw(objGFX, transparency, true, true, true, true, 0, 0, super.viewWidth, super.viewHeight);
	}



	/**
	 * Renders a partial area of the Layer using the given GFX object. <br>
	 */
	public void draw(Object objGFX, boolean transparency, int x, int y, int width, int height) {
		draw(objGFX, transparency, true, true, true, true, x, y, width, height);
	}



	/**
	 * Returns the {@link Layer} associated with this Viewer. <br>
	 * @return the Layer associated with this Viewer
	 */
	public Layer getLayer() {
		return layer;
	}



	// -------------------------------------------------------------------------
	// Private Methods
	// -------------------------------------------------------------------------

	// guess what this does!?
	private void draw(Object gfx, boolean transparency, boolean drawBackground, boolean drawForeground1, boolean drawForeground2, boolean drawForeground3, int dstPixX, int dstPixY, int dstPixWidth, int dstPixHeight) {
		int startX		= dstPixX + pixelX;
		int startY		= dstPixY + pixelY;
		int mapX 		= startX / blockGapX;
		int mapY 		= startY / blockGapY;
		dstPixX			+= super.viewOffsetX;
		dstPixY			+= super.viewOffsetY;
		dstPixWidth		+= dstPixX;		// this is the max pix X, hence = viewOffsetX + dstPixX + dstPixWidth
		dstPixHeight	+= dstPixY;

		int	mapXStart;
		int	dstXStart;
		int	imageIndex;

		int blockHeight = mapHeader.getBlockHeight();

		super.renderer.setClip(gfx, super.viewOffsetX, super.viewOffsetY, super.viewWidth, super.viewHeight);

		// initialise loop variables
		dstPixX	-= (startX % blockGapX);
		dstPixY	-= (startY % blockGapY);

		// if hex / isometric...
		boolean isIsometric = (staggerX != 0) || (staggerY != 0);
		if (isIsometric) {
			transparency = true;
			dstPixX -= staggerX;
			dstPixY -= staggerY;
			mapY *= 2;

			// all layers (pillars) need to rendered in isometric mode
			drawBackground = true;
			drawForeground1 = true;
			drawForeground2 = true;
			drawForeground3 = true;
		}

		mapXStart	 = mapX;
		dstXStart	 = dstPixX;

		// cache layer data for getBlock() inlining
		short[][] 	layerData 	= layer.getLayerData();
		Block[]		blocks		= layer.getBlocks();
		AnimBlock[] animBlocks 	= layer.getAnimBlocks();

		// loop for all blocks to be rendered
		int drawModeB 	= Renderer.NONE;
		int drawModeF 	= Renderer.NONE;
		boolean myTrans	= transparency;
		int isoBh		= pillarRiserMode ? blockHeight : 0;
		int blockIndex;
		Block block;
		if (pillarRiserMode)
			dstPixHeight += tallestPillar;
		for ( ;dstPixY < dstPixHeight; dstPixY += blockGapY) {

			mapX = mapXStart;
			for (dstPixX = dstXStart ;dstPixX < dstPixWidth; dstPixX += blockGapX) {

				if (!pillarRiserMode || (mapY < mapBlockHeight)) {
					// inlined Layer.getBlock() for speed
					blockIndex = layerData[mapY][mapX];
					if (blockIndex < 0)
						blockIndex = animBlocks[-blockIndex - 1].getCurrentFrame();
				} else
					blockIndex = 0;

				int isoY = dstPixY;
				blockIndex--;
				do {
					blockIndex++;
					block = blocks[blockIndex];

					if (pillarRiserMode) {
						if (block.getFlag(Block.LEFT_SIDE_PILLAR))
							drawModeF = Renderer.DRAW_LEFT_SIDE_ONLY;
						else if (block.getFlag(Block.RIGHT_SIDE_PILLAR))
							drawModeF = Renderer.DRAW_RIGHT_SIDE_ONLY;
						else
							drawModeF = Renderer.NONE;
						if (isoY == dstPixY) {
							drawModeB = Renderer.NONE;
							myTrans = transparency;
						} else {
							drawModeB = drawModeF;
							myTrans = true;
						}
					}

					if (drawBackground) {
						imageIndex = block.getImageIndex(Block.BACKGROUND);
						if ((!myTrans) || (imageIndex != 0))
							renderer.drawImage(gfx, imageIndex, dstPixX, isoY, myTrans, drawModeB);
					}

					isoY -= isoBh;
					if (drawForeground1) {
						imageIndex = block.getImageIndex(Block.FOREGROUND1);
						if (imageIndex != 0)
							renderer.drawImage(gfx, imageIndex, dstPixX, isoY, true, drawModeF);
					}

					isoY -= isoBh;
					if (drawForeground2) {
						imageIndex = block.getImageIndex(Block.FOREGROUND2);
						if (imageIndex != 0)
							renderer.drawImage(gfx, imageIndex, dstPixX, isoY, true, drawModeF);
					}

					isoY -= isoBh;
					if (drawForeground3) {
						imageIndex = block.getImageIndex(Block.FOREGROUND3);
						if (imageIndex != 0)
							renderer.drawImage(gfx, imageIndex, dstPixX, isoY, true, drawModeF);
					}

					isoY -= isoBh;
				} while (pillarRiserMode && block.getFlag(Block.ATTACH_NEXT));

				mapX++;
			}


			// if isometric... do the next half row
			if (isIsometric) {
				mapX = mapXStart;
				mapY++;
				dstPixY += staggerY;
				for (dstPixX = (dstXStart + staggerX); dstPixX < dstPixWidth; dstPixX += blockGapX) {

					if (!pillarRiserMode || (mapY < mapBlockHeight)) {
						// inlined Layer.getBlock() for speed
						blockIndex = layerData[mapY][mapX];
						if (blockIndex < 0)
							blockIndex = animBlocks[-blockIndex - 1].getCurrentFrame();
					} else
						blockIndex = 0;

					int isoY = dstPixY;
					blockIndex--;
					do {
						blockIndex++;
						block = blocks[blockIndex];

						if (pillarRiserMode) {
							if (block.getFlag(Block.LEFT_SIDE_PILLAR))
								drawModeF = Renderer.DRAW_LEFT_SIDE_ONLY;
							else if (block.getFlag(Block.RIGHT_SIDE_PILLAR))
								drawModeF = Renderer.DRAW_RIGHT_SIDE_ONLY;
							else
								drawModeF = Renderer.NONE;
							if (isoY == dstPixY) {
								drawModeB = Renderer.NONE;
								myTrans = transparency;
							} else {
								drawModeB = drawModeF;
								myTrans = true;
							}
						}

						if (drawBackground) {
							imageIndex = block.getImageIndex(Block.BACKGROUND);
							if ((!myTrans) || (imageIndex != 0))
								renderer.drawImage(gfx, imageIndex, dstPixX, isoY, myTrans, drawModeB);
						}

						isoY -= isoBh;
						if (drawForeground1) {
							imageIndex = block.getImageIndex(Block.FOREGROUND1);
							if (imageIndex != 0)
								renderer.drawImage(gfx, imageIndex, dstPixX, isoY, true, drawModeF);
						}

						isoY -= isoBh;
						if (drawForeground2) {
							imageIndex = block.getImageIndex(Block.FOREGROUND2);
							if (imageIndex != 0)
								renderer.drawImage(gfx, imageIndex, dstPixX, isoY, true, drawModeF);
						}

						isoY -= isoBh;
						if (drawForeground3) {
							imageIndex = block.getImageIndex(Block.FOREGROUND3);
							if (imageIndex != 0)
								renderer.drawImage(gfx, imageIndex, dstPixX, isoY, true, drawModeF);
						}

						isoY -= isoBh;
					} while (pillarRiserMode && block.getFlag(Block.ATTACH_NEXT));
					mapX++;
				}
				dstPixY -= staggerY;
			}
			mapY++;
		}

		// restore Clip
		renderer.restoreClip(gfx);
	}
}
