package com.alienfactory.javamappy.viewer;

import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.viewer.render.Renderer;



// -----------------------------------------------------------------------------
/**
 * Renders a <code>Map</code> onto the screen. A <code>MapViewer</code> creates
 * a {@link LayerViewer} for each of the <code>Layers</code> in the
 * <code>Map</code>. <br>
 * <br>
 * Updating the coordinates of a <code>MapViewer</code> also updates the
 * coordinates of all the underlying <code>LayerViewers</code>. <br>
 * <br>
 * Note that the coordinates of a <code>MapViewer</code> are held independently
 * of the coordinates held by the underlying <code>LayerViewers</code>. i.e. If
 * you were to change the coordinates of the <code>LayerViewers</code>, the
 * coordinates held by the <code>MapViewer</code> would remain unchanged. <br>
 *
 * @author Steve Eynon
 **/
// -----------------------------------------------------------------------------
public class MapViewer extends Viewer {

	private LayerViewer[]	layerViewer		= new LayerViewer[Map.MAX_NO_OF_LAYERS];
	private final Map		map;



	//--------------------------------------------------------------------------
	// Constructors
	//--------------------------------------------------------------------------

	/**
	 * Creates a {@link LayerViewer} for each of the <code>Layers</code> in the
	 * <code>Map</code>. The viewing area defaults to the top left hand corner
	 * of the screen. <br>
	 * @param map			the <code>Map</code> to be viewed
	 * @param renderer		the <code>Renderer</code> to use when drawing the map
	 * @param viewWidth		the width of the viewing area in pixels
	 * @param viewHeight	the height of the viewing area in pixels
	 */
	public MapViewer(Map map, Renderer renderer, int viewWidth, int viewHeight) {
		this(map, renderer, viewWidth, viewHeight, 0, 0);
	}



	/**
	 * Creates a {@link LayerViewer} for each of the <code>Layers</code> in the
	 * <code>Map</code> and places the viewing area at the given offset. <br>
	 * @param map			the <code>Map</code> to be viewed
	 * @param renderer		the <code>Renderer</code> to use when drawing the map
	 * @param viewWidth		the width of the viewing area in pixels
	 * @param viewHeight	the height of the viewing area in pixels
	 * @param viewOffsetX	the initial offset in pixels of the viewing area
	 * 		  				from the top left hand corner of the screen
	 * @param viewOffsetY	the initial offset in pixels of the viewing area
	 * 						from the top left hand corner of the screen
	 */
	public MapViewer(Map map, Renderer renderer, int viewWidth, int viewHeight, int viewOffsetX, int viewOffsetY) {
    	super(map.getMapHeader(), renderer, viewWidth, viewHeight, viewOffsetX, viewOffsetY);

		this.map		= map;
		for (int i=0; i<map.getLayers().length; i++)
			layerViewer[i] = new LayerViewer(map.getLayers()[i], map.getMapHeader(), renderer, viewWidth, viewHeight, viewOffsetX, viewOffsetY);
	}



	// -------------------------------------------------------------------------
	// Public Methods
	// -------------------------------------------------------------------------

	/**
	 * Returns an array of {@link LayerViewer}s. <br>
	 */
	public LayerViewer[] getLayerViewers() {
		return layerViewer;
	}



	/**
	 * Returns the <code>Map</code> associated with this viewer. <br>
	 */
	public Map getMap() {
		return map;
	}



	// JavaDoc given by Viewer
	public void setPixelX(int pixelX) {
		super.setPixelX(pixelX);

		// update the coordinates of all the layer viewers
		for (int i = 0; i < layerViewer.length; i++)
			if (layerViewer[i] != null)
				layerViewer[i].setPixelX(pixelX);
	}



	// JavaDoc given by Viewer
	public void addPixelX(int deltaX) {
		super.addPixelX(deltaX);

		// update the coordinates of all the layer viewers
		for (int i = 0; i < layerViewer.length; i++)
			if (layerViewer[i] != null)
				layerViewer[i].addPixelX(deltaX);
	}



	// JavaDoc given by Viewer
	public void setPixelY(int pixelY) {
		super.setPixelY(pixelY);

		// update the coordinates of all the layer viewers
		for (int i = 0; i < layerViewer.length; i++)
			if (layerViewer[i] != null)
				layerViewer[i].setPixelY(pixelY);
	}



	// JavaDoc given by Viewer
	public void addPixelY(int deltaY) {
		super.addPixelY(deltaY);

		// update the coordinates of all the layer viewers
		for (int i = 0; i < layerViewer.length; i++)
			if (layerViewer[i] != null)
				layerViewer[i].addPixelY(deltaY);
	}



	// JavaDoc given by Viewer
	public void setBlockX(int blockX) {
		super.setBlockX(blockX);

		// update the coordinates of all the layer viewers
		for (int i = 0; i < layerViewer.length; i++)
			if (layerViewer[i] != null)
				layerViewer[i].setBlockX(blockX);
	}



	// JavaDoc given by Viewer
	public void addBlockX(int blockX) {
		super.addBlockX(blockX);

		// update the coordinates of all the layer viewers
		for (int i = 0; i < layerViewer.length; i++)
			if (layerViewer[i] != null)
				layerViewer[i].addBlockX(blockX);
	}



	// JavaDoc given by Viewer
	public void setBlockY(int blockY) {
		super.setBlockY(blockY);

		// update the coordinates of all the layer viewers
		for (int i = 0; i < layerViewer.length; i++)
			if (layerViewer[i] != null)
				layerViewer[i].setBlockY(blockY);
	}



	// JavaDoc given by Viewer
	public void addBlockY(int blockY) {
		super.addBlockY(blockY);

		// update the coordinates of all the layer viewers
		for (int i = 0; i < layerViewer.length; i++)
			if (layerViewer[i] != null)
				layerViewer[i].addBlockY(blockY);
	}



	/**
	 * A helper method that updates the animation of all the
	 * {@link com.alienfactory.javamappy.AnimBlock AnimBlocks} in the
	 * <code>Map</code>. <br>
	 */
	public void updateAnimBlocks() {
		// this map may not have any animBlocks
		if (map.getAnimBlocks() == null) return;

		for (int index = 0; index < map.getAnimBlocks().length; index++)
			map.getAnimBlocks()[index].updateAnimation();
	}



	/**
	 * Renders the Map using the given GFX object. <br>
	 * @param objGFX the gfx object specific to the <code>Renderer</code>
	 * @param transparency whether or not the background layer of the blocks
	 * should be rendered honouring their transparent pixels
	 */
	public void draw(Object objGFX, boolean transparency) {
		for (int i = 0; i < layerViewer.length; i++)
			if (layerViewer[i] != null)
				layerViewer[i].draw(objGFX, transparency);
	}



	/**
	 * Renders a partial area of the Map using the given GFX object. <br>
	 */
	public void draw(Object objGFX, boolean transparency, int x, int y, int width, int height) {
		for (int i = 0; i < layerViewer.length; i++) {
			if (layerViewer[i] != null)
				layerViewer[i].draw(objGFX, transparency, x, y, width, height);
		}
	}



	// Javadoc given by Viewer
	public void setPillarRiserMode(boolean pillarRiserMode) {
		super.setPillarRiserMode(pillarRiserMode);

		// update the pillarRiserMode of all the layer viewers
		for (int i = 0; i < layerViewer.length; i++)
			if (layerViewer[i] != null)
				layerViewer[i].setPillarRiserMode(pillarRiserMode);
	}

}
