package com.alienfactory.javamappy.viewer;

import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.TiledLayer;

import com.alienfactory.javamappy.AnimBlock;
import com.alienfactory.javamappy.Block;
import com.alienfactory.javamappy.Layer;
import com.alienfactory.javamappy.MapHeader;
import com.alienfactory.javamappy.util.ParameterChecker;



/**
 * <p>This class creates and wraps a J2ME MIDP 2.0 <code>TiledLayer</code>
 * representing a given {@link Layer}. This class effectively bridges the gap
 * between the MIDP 2.0 and JavaMappy APIs.</p>
 *
 * @author Steve Eynon
 **/
public class TiledLayerWrapper {

	private final Layer			layer;
	private final TiledLayer	tiledLayer;

	private final int[]			animBlocks;
	private boolean				block0included;



	/**
	 * Creates a <code>TiledLayer</code> to display the given {@link Layer}
	 *
	 * @param layer the JavaMappy Layer the <code>TiledLayer</code> will display
	 * @param imageLayer the imageLayer of the Layer to display - see {@link Block#getImageIndex(int)}
	 * @param tileSheetImage the tile sheet image to create the <code>TiledLayer</code> with
	 * @param block0included set to <code>true</code> if the tile sheet image includes the blank block 0
	 **/
	 public TiledLayerWrapper(Layer layer, int imageLayer, Image tileSheetImage, boolean block0included) {
    	ParameterChecker.checkNotNull(layer, "layer");
    	ParameterChecker.checkNotNull(tileSheetImage, "tileSheetImage");

		this.layer = layer;
		this.block0included = block0included;

		MapHeader mapHeader = layer.getMapHeader();
		int columns 	= mapHeader.getMapBlockWidth();
		int rows 		= mapHeader.getMapBlockHeight();
		int blockWidth	= mapHeader.getBlockWidth();
		int blockHeight	= mapHeader.getBlockHeight();

		tiledLayer = new TiledLayer(columns, rows, tileSheetImage, blockWidth, blockHeight);

		AnimBlock[] realAnimBlocks = layer.getAnimBlocks();
		if (realAnimBlocks == null)
			realAnimBlocks = new AnimBlock[0];

		animBlocks = new int[realAnimBlocks.length];
		for (int i = 0; i < realAnimBlocks.length; i++) {
			int tileIndex = tiledLayer.createAnimatedTile(realAnimBlocks[i].getCurrentFrame());
			animBlocks[i] = tileIndex;
		}

		int tileIndex = 0;
		short[][] map = layer.getLayerData();
		for (int y = 0; y < map.length; y++)
			for (int x = 0; x < map[y].length; x++) {

				if (layer.isAnimBlock(x, y)) {
					int blockIndex = layer.getBlockIndex(x, y);
					tileIndex = animBlocks[-blockIndex - 1];

				} else {
					tileIndex = layer.getBlock(x, y).getImageIndex(imageLayer);
					tileIndex += block0included ? 1 : 0;
				}

				tiledLayer.setCell(x, y, tileIndex);
			}
	}



	/**
	 * Returns the {@link Layer} associated with this Viewer. <br>
	 * @return the Layer associated with this Viewer
	 */
	public Layer getLayer() {
		return layer;
	}



	/**
	 * Returns the {@link TiledLayer} the class wraps. <br>
	 * @return the wrapped TiledLayer
	 */
	public TiledLayer getTiledLayer() {
		return tiledLayer;
	}


	/**
	 * <p>Applies the current state of the AnimBlocks to the TiledLayer. Typically
	 * this would be called after updating the AnimBlocks. For example:</p>
	 * <pre>
	 * mapViewer.updateAnimBlocks();
	 * tiledLayerWrapper.animateLayer();
	 * </pre>
	 * @see MapViewer#updateAnimBlocks()
	 */
	public void animateLayer() {
		AnimBlock[] realAnimBlocks = layer.getAnimBlocks();

		if (realAnimBlocks == null)
			return;

		for (int i = 0; i < realAnimBlocks.length; i++) {
			realAnimBlocks[i].updateAnimation();

			int
			tileIndex = realAnimBlocks[i].getCurrentFrame();
			tileIndex += block0included ? 1 : 0;

			tiledLayer.setAnimatedTile(animBlocks[i], tileIndex);
		}
	}
}
