package com.alienfactory.javamappy.viewer;

import com.alienfactory.javamappy.MapHeader;
import com.alienfactory.javamappy.util.ParameterChecker;
import com.alienfactory.javamappy.viewer.render.Renderer;


// -----------------------------------------------------------------------------
/**
 * Renders a <code>Map</code> or a <code>Layer</code> (dependent on the concrete
 * implementation) onto the screen. By default the map is rendered to the top
 * left hand corner of the screen as shown below: <br/>
 *
 * <pre>
 *    +------------------+---------+
 *    |                  |         |
 *    | JavaMappy Viewer |         |
 *    |                  |         |
 *    |                  |         |
 *    +------------------+         |
 *    |                            |
 *    | Screen                     |
 *    |                            |
 *    +----------------------------+ </pre>
 *
 * (A <code>Viewer</code> with a width and height smaller than the screen) <br/>
 * <br>
 *
 * However, you can move the viewing area around the screen by specifying an X
 * and Y offset as shown below: <br/>
 *
 * <pre>
 *    +----------------------------+
 *    | Screen                     |
 *    |    +------------------+    |
 *    |    |                  |    |
 *    |    | JavaMappy Viewer |    |
 *    |    |                  |    |
 *    |    |                  |    |
 *    |    +------------------+    |
 *    |                            |
 *    +----------------------------+ </pre>
 *
 * (The same <code>Viewer</code> with an X and Y offset) <br/>
 *
 * @author Steve Eynon
 **/
// -----------------------------------------------------------------------------
public abstract class Viewer {

    protected final MapHeader		mapHeader;
    protected final Renderer 		renderer;
    protected int	viewWidth, 		viewHeight;
    protected int 	viewOffsetX,	viewOffsetY;
    protected int	pixelX, 		pixelY;
   	protected boolean				pillarRiserMode;



	// -------------------------------------------------------------------------
	// Constructors
	// -------------------------------------------------------------------------

	/**
	 * Creates a <code>Viewer</code> and places the viewing area at the given
	 * offset. <br>
	 * @param mapHeader		contains useful information
	 * @param renderer		the <code>Renderer</code> to use when drawing
	 * @param viewWidth		the width of the viewing area in pixels
	 * @param viewHeight	the height of the viewing area in pixels
	 * @param viewOffsetX	the initial offset in pixels of the viewing area
	 * 		  				from the top left hand corner of the screen
	 * @param viewOffsetY	the initial offset in pixels of the viewing area
	 * 						from the top left hand corner of the screen
	 */
    public Viewer(MapHeader mapHeader, Renderer renderer, int viewWidth, int viewHeight, int viewOffsetX, int viewOffsetY) throws IllegalArgumentException {
    	ParameterChecker.checkNotNull(mapHeader, "mapHeader");
    	ParameterChecker.checkNotNull(renderer, "renderer");

		this.mapHeader	= mapHeader;
		this.renderer	= renderer;

		setViewWidth	(viewWidth);
		setViewHeight	(viewHeight);
		setViewOffsetX	(viewOffsetX);
		setViewOffsetY	(viewOffsetY);

		pillarRiserMode	= mapHeader.isRisingPillarMode();
    }



	// -------------------------------------------------------------------------
	// Public Getters and Setters
	// -------------------------------------------------------------------------

	/**
	 * Returns the width in pixels of the viewing area. <br>
	 */
	public int getViewWidth() {
		return viewWidth;
	}



	/**
	 * Sets the width in pixels of the viewing area. <br>
	 * @throws IllegalArgumentException if viewWidth is less than or equal to zero
	 */
	public void setViewWidth(int viewWidth) throws IllegalArgumentException {
		ParameterChecker.checkMinMax(viewWidth, "viewWidth", 1, mapHeader.getMapPixelWidth());
		this.viewWidth = viewWidth;
	}



	/**
	 * Returns the height in pixels of the viewing area. <br>
	 */
	public int getViewHeight() {
		return viewHeight;
	}



	/**
	 * Sets the height in pixels of the viewing area. <br>
	 * @throws IllegalArgumentException if viewWidth is less than or equal to zero
	 */
	public void setViewHeight(int viewHeight) throws IllegalArgumentException {
		ParameterChecker.checkMinMax(viewHeight, "viewHeight", 1, mapHeader.getMapPixelHeight());
		this.viewHeight = viewHeight;
	}



	/**
	 * Returns the horizontal offset in pixels of the viewing area from the top
	 * left hand corner of the screen. <br>
	 */
	public int getViewOffsetX() {
		return viewOffsetX;
	}



	/**
	 * Sets the horizontal offset in pixels of the viewing area from the top
	 * left hand corner of the screen. <br>
	 * @throws IllegalArgumentException if viewOffsetX is less than zero
	 */
	public void setViewOffsetX(int viewOffsetX) throws IllegalArgumentException {
		ParameterChecker.checkMin(viewOffsetX, "viewOffsetX", 0);
		this.viewOffsetX = viewOffsetX;
	}



	/**
	 * Returns the vertical offset in pixels of the viewing area from the top
	 * left hand corner of the screen. <br>
	 */
	public int getViewOffsetY() {
		return viewOffsetY;
	}



	/**
	 * Sets the vertical offset in pixels of the viewing area from the top left
	 * hand corner of the screen. <br>
	 * @throws IllegalArgumentException if viewOffsetX is less than zero
	 */
	public void setViewOffsetY(int viewOffsetY) throws IllegalArgumentException {
		ParameterChecker.checkMin(viewOffsetY, "viewOffsetY", 0);
		this.viewOffsetY = viewOffsetY;
	}


	/**
	 * Returns the {@link Renderer} associated with this Viewer. This method
	 * should not return <code>null</code>. <br>
	 * @return the Renderer associated with this Viewer
	 */
	public Renderer getRenderer() {
		return renderer;
	}



	/**
	 * Returns the {@link MapHeader} associated with this Viewer. This method
	 * should not return <code>null</code>. <br>
	 * @return the MapHeader associated with this Viewer
	 */
	public MapHeader getMapHeader() {
		return mapHeader;
	}



	/**
	 * Returns <code>true</code> is this viewer is in Pillar Riser Mode, used in
	 * isometric maps. <br>
	 */
	public boolean isPillarRiserMode() {
		return pillarRiserMode;
	}



	/**
	 * Sets the Pillar Riser Mode, as used in isometric maps. <br>
	 */
	public void setPillarRiserMode(boolean pillarRiserMode) {
		this.pillarRiserMode = pillarRiserMode;
	}



	// -------------------------------------------------------------------------
	// Public Coordinate Methods
	// -------------------------------------------------------------------------

	/**
	 * Returns the X coordinate in pixels of the top left hand corner of the
	 * viewing area. <br>
	 * @return the X coordinate in pixels
	 */
	public int getPixelX() {
		return pixelX;
	}



	/**
	 * Sets the X coordinate in pixels of the top left hand corner of the
	 * viewing area. <br>
	 * @param pixelX the new X coordinate in pixels
	 */
	public void setPixelX(int pixelX) {
		ParameterChecker.checkMinMax(pixelX, "pixelX", 0, mapHeader.getMapPixelWidth() - viewWidth);
		this.pixelX = pixelX;
	}



	/**
	 * Increments the X coordinate by the given pixel amount. Negative values
	 * are allowed. <br>
	 * @param deltaX the amount to add to the X coordinate in pixels
	 */
	public void addPixelX(int deltaX) {
		this.pixelX += deltaX;
	}



	/**
	 * Returns the Y coordinate in pixels of the top left hand corner of the
	 * viewing area. <br>
	 * @return the Y coordinate in pixels
	 */
	public int getPixelY() {
		return pixelY;
	}



	/**
	 * Sets the Y coordinate in pixels of the top left hand corner of the
	 * viewing area. <br>
	 * @param pixelY the new Y coordinate in pixels
	 */
	public void setPixelY(int pixelY) {
		ParameterChecker.checkMinMax(pixelY, "pixelY", 0, mapHeader.getMapPixelHeight() - viewHeight);
		this.pixelY = pixelY;
	}



	/**
	 * Increments the Y coordinate by the given pixel amount. Negative values
	 * are allowed. <br>
	 * @param deltaY the amount to add to the Y coordinate in pixels
	 */
	public void addPixelY(int deltaY) {
		this.pixelY += deltaY;
	}



	/**
	 * Returns the X coordinate in blocks of the top left hand corner of the
	 * viewing area. <br>
	 * @return the X coordinate in blocks
	 */
	public int getBlockX() {
		return pixelX / mapHeader.getBlockWidth();
	}



	/**
	 * Sets the X coordinate in blocks of the top left hand corner of the
	 * viewing area. <br>
	 * @param blockX the new X coordinate in blocks
	 */
	public void setBlockX(int blockX) {
		ParameterChecker.checkMinMax(blockX, "blockX", 0, mapHeader.getMapBlockWidth() - (viewWidth / mapHeader.getBlockWidth()));
		pixelX = blockX * mapHeader.getBlockWidth();
	}



	/**
	 * Increments the X coordinate by the given block amount. Negative values
	 * are allowed. <br>
	 * @param blockX the amount to add to the X coordinate in blocks
	 */
	public void addBlockX(int blockX) {
		pixelX += blockX * mapHeader.getBlockWidth();
	}



	/**
	 * Returns the Y coordinate in blocks of the top left hand corner of the
	 * viewing area. <br>
	 * @return the Y coordinate in blocks
	 */
	public int getBlockY() {
		return pixelY / mapHeader.getBlockHeight();
	}



	/**
	 * Sets the Y coordinate in blocks of the top left hand corner of the
	 * viewing area. <br>
	 * @param blockY the new Y coordinate in blocks
	 */
	public void setBlockY(int blockY) {
		ParameterChecker.checkMinMax(blockY, "blockY", 0, mapHeader.getMapBlockHeight() - (viewHeight / mapHeader.getBlockHeight()));
		pixelY = blockY * mapHeader.getBlockHeight();
	}



	/**
	 * Increments the Y coordinate by the given block amount. Negative values
	 * are allowed. <br>
	 * @param blockY the amount to add to the Y coordinate in blocks
	 */
	public void addBlockY(int blockY) {
		pixelY += blockY * mapHeader.getBlockHeight();
	}



	// -------------------------------------------------------------------------
	// Public Abstract Methods
	// -------------------------------------------------------------------------

	/**
	 * Draws Mappy data into the viewing area. <br>
	 * @param objGFX the gfx object specific to the <code>Renderer</code>
	 * @param transparency whether or not the background layer of the blocks
	 * should be rendered honouring their transparent pixels
	 */
	public abstract void draw(Object objGFX, boolean transparency);


	/**
	 * Draws but just a section of the viewing area. To be used when you don't
	 * need to render the whole screen, but just part of it. <br>
	 * @param x the pixel x coordinate from the left hand side of the viewing area
	 * @param y the pixel y coordinate from the top of the viewing area
	 * @param width the width, in pixels, of the viewing area to render
	 * @param height the height, in pixels, of the viewing area to render
	 */
	public abstract void draw(Object objGFX, boolean transparency, int x, int y, int width, int height);
}
