package com.alienfactory.javamappy.viewer.render;

import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;

import javax.imageio.ImageIO;

import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.PixelData;
import com.alienfactory.javamappy.util.ParameterChecker;



// -----------------------------------------------------------------------------
/**
 * This <code>Renderer</code> has been optimised for use in Java2 SE 1.4 JVMs or
 * later. <br>
 *
 * @author Steve Eynon
 **/
// -----------------------------------------------------------------------------
public class J2SE14Renderer extends JDKRenderer {

	/**
	 * Uses the raw pixel data and colour map from the given <code>Map</code> to
	 * create images in a JDK specific manner. <br>
	 * <br>
	 * @param map the {@link Map} containing the image data to be rendered
	 * @throws IllegalArgumentException if <code>Map</code> is <code>null</code>
	 **/
	public J2SE14Renderer(Map map) throws IllegalArgumentException {
		super(map);

		PixelData imageData[]	= map.getImageData();
		super.opaqueImages		= new Image[imageData.length];
		super.transparentImages	= new Image[imageData.length];

		GraphicsEnvironment		gfxEnviro = GraphicsEnvironment.getLocalGraphicsEnvironment();
		GraphicsDevice			gfxDevice = gfxEnviro.getDefaultScreenDevice();
	    GraphicsConfiguration 	gfxConfig = gfxDevice.getDefaultConfiguration();

	    Graphics 				gfx;
	    PixelData				pixelData;
		BufferedImage 			image;

		for (int i=0; i < imageData.length; i++) {
			pixelData		= imageData[i];

			image = new BufferedImage(blockWidth, blockHeight, BufferedImage.TYPE_INT_ARGB);
			image.setRGB(0, 0, blockWidth, blockHeight, imageData[i].getRawPixelData(), 0, blockWidth);

			// draw our opaque image
			super.opaqueImages[i] = gfxConfig.createCompatibleImage(blockWidth, blockHeight);
			gfx = super.opaqueImages[i].getGraphics();
			gfx.drawImage(image, 0, 0, null);
			gfx.dispose();

			// don't bother making a a transparent image if it has no transparent pixels
			if (pixelData.hasTransparentPixels()) {
				super.transparentImages[i] = gfxConfig.createCompatibleImage(blockWidth, blockHeight, Transparency.BITMASK);
				gfx = super.transparentImages[i].getGraphics();
				gfx.drawImage(image, 0, 0, null);
				gfx.dispose();
			}

			// Free unused graphic resources.
			image.flush();
        }
	}


	/**
	 * Loads the tile sheet image from the given Input Stream (which is not
	 * closed). The image can be of the usual formats, .BMP, .GIF, .PNG, .JPG,
	 * etc. <br>
	 * <br>
	 * Useful when working with .fma files so the same tile sheet can be used
	 * with multiple maps. <br>
	 * <br>
	 * @param map the {@link Map} containing the image data to be rendered
	 * @param tileSheet the input stream to load the tile image from
	 * @param block0included set to <code>true</code> if the tile image includes block 0
	 * @throws IllegalArgumentException if input parameter is <code>null</code>
	 **/
	public J2SE14Renderer(Map map, InputStream tileSheet, boolean block0included) throws IOException, IllegalArgumentException {
		super(map);
		ParameterChecker.checkNotNull(tileSheet, "tileSheet");

		// this is the 1.4 specific line
		BufferedImage imageMap = ImageIO.read(tileSheet);

		int noOfTilesInRow = imageMap.getWidth()  / blockWidth;
		int noOfTilesInCol = imageMap.getHeight() / blockHeight;
		int noOfTiles = (noOfTilesInRow * noOfTilesInCol) + (block0included ? 0 : 1);
		super.opaqueImages = super.transparentImages = new Image[noOfTiles];

		int i = (block0included ? 0 : 1);
		BufferedImage buffImage;
		Graphics gfx;
		for (int y = 0; y < noOfTilesInCol; y++) {
			for (int x = 0; x < noOfTilesInRow; x++) {
				buffImage = new BufferedImage(blockWidth, blockHeight, BufferedImage.TYPE_INT_ARGB);
				gfx = buffImage.getGraphics();
				gfx.drawImage(imageMap, -(x * blockWidth), -(y * blockHeight), null);
				gfx.dispose();
				super.transparentImages[i++] = buffImage;
			}
		}
	}
}
