package com.alienfactory.javamappy.viewer.render;

import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.IndexColorModel;
import java.awt.image.MemoryImageSource;

import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.MapHeader;
import com.alienfactory.javamappy.PixelData;
import com.alienfactory.javamappy.util.Logger;



// -----------------------------------------------------------------------------
/**
 * This <code>Renderer</code> is for use by JDK 1.1.x JVMs. This is the only
 * Renderer suitable for use by all Applets (including Microsoft's JVM). <br>
 * <br>
 * The <code>JDK11Renderer</code> creates its images using
 * <code>java.awt.image.MemoryImageSources</code>. If the map has a colour depth
 * of 8 then <code>java.awt.image.IndexColorModels</code> are used to speed up
 * image rendering. (Thanks go to Robin Burrows for the tip) <br>
 *
 * @author Steve Eynon
 */
// -----------------------------------------------------------------------------
public class JDK11Renderer extends JDKRenderer {
	private Logger LOG = Logger.createLogger(this.getClass());



	/**
	 * Uses the raw pixel data and colour map from the given <code>Map</code> to
	 * create the solid and transparent images in a JDK specific manner. <br>
	 * <br>
	 * @param map the {@link Map} containing the image data to be rendered
	 * @throws IllegalArgumentException if <code>Map</code> is <code>null</code>
	 **/
	public JDK11Renderer(Map map) throws IllegalArgumentException {
		super(map);

		PixelData imageData[]	= map.getImageData();
		MapHeader mapHeader 	= map.getMapHeader();
		int[]  	colours 		= map.getColours();

		super.opaqueImages		= new Image[imageData.length];
		super.transparentImages	= new Image[imageData.length];

		int[] intImageData;
		PixelData pixelData;

		// index colour models are only valid for Colour Map images
		boolean usesColourMap = (mapHeader.getColourDepth() == 8);
		IndexColorModel icmSolid = null, icmTrans = null;
		if (usesColourMap) {
		    LOG.debug("Using IndexColourModels");
			icmSolid = new IndexColorModel(8, colours.length, toByteArray(colours), 0, false);
			icmTrans = new IndexColorModel(8, colours.length, toByteArray(colours), 0, false, mapHeader.getChromeKeyIndex());
		}

		for (int i=0; i < imageData.length; i++) {
			pixelData		= imageData[i];

			// create the opaque image
			MemoryImageSource mis;
			if (usesColourMap)
				mis = new MemoryImageSource(blockWidth, blockHeight, icmSolid, pixelData.getCmapPixelData(), 0, blockWidth);
			else
				mis = new MemoryImageSource(blockWidth, blockHeight, pixelData.getRawPixelData(), 0, blockWidth);
			super.opaqueImages[i] = Toolkit.getDefaultToolkit().createImage(mis);

			if (pixelData.hasTransparentPixels()) {
				if (usesColourMap) {
					super.transparentImages[i] = Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(blockWidth, blockHeight, icmTrans, pixelData.getCmapPixelData(), 0, blockWidth));

				} else {
					// in JDK1.1 the MemoryImageSource doesn't copy the given
					// array, so any changes you make to the array are reflected
					// in the generated image!
					intImageData = new int[pixelData.getRawPixelData().length];
					System.arraycopy(pixelData.getRawPixelData(), 0, intImageData, 0, intImageData.length);
					super.transparentImages[i] = Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(blockWidth, blockHeight, intImageData, 0, blockWidth));
				}
			}
		}
	}



	/**
	 * Returns the <code>Colour[]</code> as an array of <code>bytes</code>. <br>
	 */
	private static byte[] toByteArray(int[] colours) {
		byte[] bytColours = new byte[colours.length * 3];

		// convert the colour objects into bytes
		int index = 0;
		for (int i = 0; i < colours.length; i++) {
			bytColours[index++] = (byte) ((colours[i] >> 16) & 0xFF);
			bytColours[index++] = (byte) ((colours[i] >>  8) & 0xFF);
			bytColours[index++] = (byte) ((colours[i] >>  0) & 0xFF);
		}

		return bytColours;
	}
}