package com.alienfactory.javamappy.viewer.render;

import java.io.IOException;

import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.util.ParameterChecker;

// -----------------------------------------------------------------------------
/**
 * An alternative J2ME Renderer to use with MIDP v1.0 compatible devices. This
 * implementation stores the tiles as an array of Images rather than a single
 * tile sheet image. Even though it uses up more memory this can help overcome
 * heap fragmentation problems caused by a large tile sheet (i.e. not having a
 * large enough continuous block of memory to accommodate the image). <br>
 * <br>
 * Consider using if you get <code>OutOfMemoryErrors</code> when trying to
 * load your tile sheet but <code>Runtime.freeMemory()</code> reckons you
 * should have enough. Also consider using if developing for Motorola V-series
 * phones as these have notorious heap fragmentation problems. <br>
 * <br>
 * Confused? Here is an extract from <code>Motocoder.com</code> in the
 * <code>Knowledge & Support -> FAQs</code> section. (See <a
 * href="http://motocoder.custhelp.com/cgi-bin/motocoder.cfg/php/enduser/std_adp.php?p_faqid=454">http://motocoder.custhelp.com/cgi-bin/motocoder.cfg/php/enduser/std_adp.php?p_faqid=454</a>,
 * login required): <br>
 * <hr>
 * You are right in the assumption of heap fragmentation. This is what is
 * causing the issue. Also the MIDlet jar is stored as a jar in the device and
 * not as independent class files. The amount of freeMem() returned in the total
 * unallocated memory not a contiguous block of memory and so when an attempt to
 * place a file (image) on memory that is large than the largest continuous
 * block a out of memory exception is thrown. <br>
 * <br>
 * We have made improvements in later models, i.e E398, etc with fragmentation.
 * However in some cases these errors are still seen if precautions are not
 * taken with memory hungry actions. <br>
 * <br>
 * It should be noted that Image object for image 640x480 will take about 615KB
 * (640 * 480 * 2 = 614400). It is strongly recommended to release Image objects
 * if they are not used any more. Our kvm implementation contains a compacting
 * Garbage Collector and it will help minimise heap memory fragmentation. <br>
 * <br>
 * Garbage is collected when a new object is allocated and there is no memory in
 * the heap to allocate this object. It can be because the memory is full or
 * because of fragmentation reasons. In this case GC is called implicitly. In
 * addition, GC can be called explicitly via System.gc(). GC dos not reclaim old
 * objects. Only not referenced objects are reclaimed <br>
 * <br>
 * When the user selects the MIDlet to run, the kvm reads the necessary classes
 * and resources from the JAR file and starts the MIDlet. Classes are loaded as
 * needed into the heap for execution, and some of the heap is reserved for
 * caching record store information. The most heap-consuming process is the
 * creation of images. For example, decoding a PNG image requires: temporary
 * buffer to read image from the JAR file + buffer for decoded image data equal
 * to size of (image width * image height * 2) bytes + about 35KB for temporary
 * PNG decoder buffers. Both temporary buffers will be released when process of
 * decoding will be finished. <br>
 * <hr>
 * Thanks go to Leo Au for bringing this to my attention. <br>
 *
 * @author Steve Eynon
 */
// -----------------------------------------------------------------------------
public class MIDP10RendererAlt extends J2MERenderer {

	private final Image[] tiles;
	private final boolean block0included;



	/**
	 * Initialises itself from the given array of tile sheet images. <br>
	 * <br>
	 * To use this constructor you must save out each tile as an individual image.
	 * <br>
	 * @param map the {@link Map} to be rendered
	 * @param tileSheetFileNames file names of the tile images to load - they should all start with a leading '/'
	 * @param block0included set to <code>true</code> if the tile sheet array includes block 0
	 */
	public MIDP10RendererAlt(Map map, String[] tileSheetFileNames, boolean block0included) throws IOException, IllegalArgumentException {
		super(map);
		ParameterChecker.checkNotNull(tileSheetFileNames, "tileSheetFileNames");

		this.block0included = block0included;
		int noOfTiles = tileSheetFileNames.length + (block0included ? 0 : 1);
		this.tiles = new Image[noOfTiles];
		int tileIndex = (block0included ? 0 : 1);
		for (int i = 0; i < tileSheetFileNames.length; i++)
			tiles[tileIndex++] = Image.createImage(tileSheetFileNames[i]);

		// destroy all objects that won't be used
		map.setColours(null);
		map.setImageData(null);
	}



	// -------------------------------------------------------------------------
	// Abstract Renderer Methods
	// -------------------------------------------------------------------------

	public void drawImage(Object objGFX, int imageIndex, int dstPixX, int dstPixY, boolean transparency, int modification) throws IllegalArgumentException {
		final Graphics gfx = (Graphics) objGFX;

		// can't draw tile image 0 if it doesn't exist!
		if (!block0included && (imageIndex == 0)) return;

		// no need to reset the clip 'cos the renderer will do that for us
		switch (modification) {
		case Renderer.NONE:
			gfx.setClip(dstPixX, dstPixY, blockWidth, blockHeight);
			break;

		case Renderer.DRAW_LEFT_SIDE_ONLY:
			gfx.setClip(dstPixX, dstPixY, halfBlockWidth, blockHeight);
			break;

		case Renderer.DRAW_RIGHT_SIDE_ONLY:
			gfx.setClip(dstPixX + halfBlockWidth, dstPixY, halfBlockWidth, blockHeight);
			break;
		}

		// draw the tile image
		gfx.drawImage(tiles[imageIndex], dstPixX, dstPixY, Graphics.TOP | Graphics.LEFT);
	}
}
