package com.alienfactory.javamappy.viewer.render;

import java.io.IOException;

import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.PixelData;
import com.alienfactory.javamappy.util.ParameterChecker;

//-----------------------------------------------------------------------------
/**
 * A <code>Renderer</code> for MIDP v2.0 compatible devices. <br>
 *
 * @author Steve Eynon
 */
// -----------------------------------------------------------------------------
public class MIDP20Renderer extends J2MERenderer {

	private final Image tileSheet;
	private final int widthInBlocks;
	private final boolean block0included;



	// -------------------------------------------------------------------------
	// Constructors
	// -------------------------------------------------------------------------

	/**
	 * Uses the raw pixel data and colour map from the given <code>Map</code> to
	 * create the tilemap image. <br>
	 * <br>
	 * For J2ME optimisation, after initialisation this Renderer clears the
	 * <code>PixelData</code> and <code>Colours</code> objects in the given
	 * <code>Map</code>. <br>
	 * <br>
	 * @param map the {@link Map} containing the image data to be rendered
	 * @throws IllegalArgumentException if <code>Map</code> is <code>null</code>
	 **/
	public MIDP20Renderer(Map map) throws IllegalArgumentException {
		super(map);

		PixelData imageData[] = map.getImageData();
		int blockDataSize = blockWidth * blockHeight;
		int[] rgbData = new int[blockDataSize * imageData.length];
		int index = 0;
		for (int i=0; i<imageData.length; i++) {
			index += blockDataSize;
			System.arraycopy(imageData[i].getRawPixelData(), 0, rgbData, index, blockDataSize);
		}

		tileSheet 		= Image.createRGBImage(rgbData, blockWidth, blockHeight * imageData.length, true);
		block0included	= true;
		widthInBlocks 	= tileSheet.getWidth() / blockWidth;

		// destroy all objects that aren't needed anymore
		map.setColours(null);
		map.setImageData(null);
	}



	/**
	 * Initialises itself from the given tileset image. Useful when loading .fma
	 * files so the same tile image may be used for multiple maps. Also useful
	 * because a .fma + .png is a lot smaller than a .fmp. <br>
	 * <br>
	 * Note that, despite the MIDP specification and only if your device
	 * supports it, transparent areas of the tilemap will be rendered as
	 * transparent. Please bear in mind that transparent rendering can increase
	 * the rendering time. <br>
	 * <br>
	 * @param map the {@link Map} to be rendered
	 * @param tileSheetFileName the fileName of the tileImage to load - should start with a leading '/'
	 * @param block0included set to <code>true</code> if the tile image includes block 0
	 */
	public MIDP20Renderer(Map map, String tileSheetFileName, boolean block0included) throws IOException, IllegalArgumentException {
		super(map);
		ParameterChecker.checkNotNull(tileSheetFileName, "tileSheetFileName");

		this.tileSheet 		= Image.createImage(tileSheetFileName);
		this.widthInBlocks 	= tileSheet.getWidth() / blockWidth;
		this.block0included	= block0included;

		// destroy all objects that won't be used
		map.setColours(null);
		map.setImageData(null);
	}



	// -------------------------------------------------------------------------
	// Abstract Renderer Methods
	// -------------------------------------------------------------------------

	/**
	 * Renders a tile image at the given coordinates. Marked <code>final</code>
	 * as a compiler optimisation. <br>
	 * <br>
	 * @param objGFX an instance of <code>javax.microedition.lcdui.Graphics</code>
	 * @param transparency is ignored
	 */
	public final void drawImage(Object objGFX, int imageIndex, int dstPixX, int dstPixY, boolean transparency, int modification) throws IllegalArgumentException {
		final Graphics	gfx		= (Graphics) objGFX;

		// compensate for altered tilemaps
		if (!block0included) {
			if (imageIndex == 0)
				return;
			imageIndex--;
		}

		// no need to reset the clip 'cos the renderer will do that for us
		switch (modification) {
		case Renderer.NONE:
			gfx.setClip(dstPixX, dstPixY, blockWidth, blockHeight);
			break;

		case Renderer.DRAW_LEFT_SIDE_ONLY:
			gfx.setClip(dstPixX, dstPixY, halfBlockWidth, blockHeight);
			break;

		case Renderer.DRAW_RIGHT_SIDE_ONLY:
			gfx.setClip(dstPixX + halfBlockWidth, dstPixY, halfBlockWidth, blockHeight);
			break;
		}

		// draw the tile image
		dstPixX -= (imageIndex % widthInBlocks) * blockWidth;
		dstPixY -= (imageIndex / widthInBlocks) * blockHeight;
		gfx.drawImage(tileSheet, dstPixX, dstPixY, Graphics.TOP | Graphics.LEFT);
	}

}
