package com.alienfactory.javamappy.viewer.render;

import java.io.IOException;

import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.PixelData;
import com.alienfactory.javamappy.util.ParameterChecker;
import com.nokia.mid.ui.DirectGraphics;
import com.nokia.mid.ui.DirectUtils;



// -----------------------------------------------------------------------------
/**
 * This <code>Renderer</code> references Nokia's own J2ME API which, unlike the
 * MIDP 1.0 specification, supports full alpha transparency. <br>
 * <br>
 * If your tile map image is being loaded from a .png then I would recommend the
 * MIDP 1.0 renderer as it still renderers transparent .pngs but is 3 to 5 times
 * faster (as tested on my own N-Gage). <br>
 *
 * @author Steve Eynon
 **/
// -----------------------------------------------------------------------------
public class NokiaRenderer extends J2MERenderer {

	/**
	 * We use a single image here instead of an array because for 28 32x32
	 * images loaded from a .FMP, it uses less than half the memory!
	 */
	private final Image image;
	private final int widthInBlocks;
	private final boolean block0included;



	// -------------------------------------------------------------------------
	// Constructors
	// -------------------------------------------------------------------------

	/**
	 * Uses the raw pixel data and colour map from the given <code>Map</code> to
	 * create the tilemap image. <br>
	 * <br>
	 * For J2ME optimisation, after initialisation this Renderer clears the
	 * <code>PixelData</code> and <code>Colours</code> objects in the given
	 * <code>Map</code>. <br>
	 * <br>
	 * @param map the {@link Map} containing the image data to be rendered
	 * @throws IllegalArgumentException if <code>Map</code> is <code>null</code>
	 **/
	public NokiaRenderer(Map map) throws IllegalArgumentException {
		super(map);
		PixelData imageData[]	= map.getImageData();

		block0included	= true;
		image 			= Image.createImage(blockWidth, blockHeight * imageData.length);
		widthInBlocks 	= image.getWidth() / blockWidth;
		DirectGraphics gfx = DirectUtils.getDirectGraphics(image.getGraphics());

		int[]			rawPixels;
		int	pixIndex	= 0;

		for (int y = 0; y < image.getHeight(); y += blockHeight) {
			rawPixels	= imageData[pixIndex++].getRawPixelData();
			gfx.drawPixels(rawPixels, false, 0, blockWidth, 0, y, blockWidth, blockHeight, 0, DirectGraphics.TYPE_INT_8888_ARGB);
		}

		// destroy all objects that aren't needed anymore
		map.setColours(null);
		map.setImageData(null);
	}



	/**
	 * Initialises itself from the given tileset image. Useful when loading .fma
	 * files so the same tileset may be used for multiple maps. Also useful
	 * because a .fma + .png is a lot smaller than a .fmp. <br>
	 * <br>
	 * Note that the tileset image needs to include image 0. <br>
	 * <br>
	 * @param map data from the <code>Map</code> is used to generate the individual images
	 * @param fileName the fileName of the image to load - should start with a leading '/'
	 * @param block0included set to <code>true</code> if the image includes block 0
	 */
	public NokiaRenderer(Map map, String fileName, boolean block0included) throws IOException, IllegalArgumentException {
		super(map);
		ParameterChecker.checkNotNull(fileName, "fileName");

		this.image 			= Image.createImage(fileName);
		this.widthInBlocks 	= image.getWidth() / blockWidth;
		this.block0included	= block0included;

		// destroy all objects that won't be used
		map.setColours(null);
		map.setImageData(null);
	}



	// -------------------------------------------------------------------------
	// Abstract Renderer Methods
	// -------------------------------------------------------------------------

	/**
	 * Renders a tile image at the given coordinates. Marked <code>final</code>
	 * as a compiler optimisation. <br>
	 * <br>
	 * @param objGFX an instance of <code>javax.microedition.lcdui.Graphics</code>
	 * @param transparency is ignored
	 */
	public final void drawImage(Object objGFX, int imageIndex, int dstPixX, int dstPixY, boolean transparency, int modification) throws IllegalArgumentException {
		Graphics gfx = (Graphics) objGFX;
		DirectGraphics dGfx = DirectUtils.getDirectGraphics(gfx);

		// compensate for altered tilemaps
		if (!block0included) {
			if (imageIndex == 0)
				return;
			imageIndex--;
		}

		dstPixX -= (imageIndex % widthInBlocks) * blockWidth;
		dstPixY -= (imageIndex / widthInBlocks) * blockHeight;

		// no need to reset the clip 'cos the renderer will do that for us
		switch (modification) {
		case Renderer.NONE:
			gfx.setClip(dstPixX, dstPixY, blockWidth, blockHeight);
			break;

		case Renderer.DRAW_LEFT_SIDE_ONLY:
			gfx.setClip(dstPixX, dstPixY, halfBlockWidth, blockHeight);
			break;

		case Renderer.DRAW_RIGHT_SIDE_ONLY:
			gfx.setClip(dstPixX + halfBlockWidth, dstPixY, halfBlockWidth, blockHeight);
			break;
		}

		dGfx.drawImage(image, dstPixX, dstPixY , Graphics.TOP | Graphics.LEFT, 0);
	}
}
