package com.alienfactory.javamappy.viewer.render;

import com.alienfactory.javamappy.Map;



// ------------------------------------------------------------------------------
/**
 * A <code>Renderer</code> contains API specific drawing routines. <br>
 *
 * @author Steve Eynon
 **/
// ------------------------------------------------------------------------------
public interface Renderer {

	/**
	 * When drawing an image this value specifies that no modifications should
	 * be applied. <br>
	 * <br>
	 * Has a constant value of <code>0</code>.
	 * @see		#drawImage(Object, int, int, int, boolean, int)
	 **/
	public static final int NONE					= 0;

	/**
	 * When drawing an image this value specifies that only the left hand side
	 * of the image should be drawn (used in Pillar Rising Mode). <br>
	 * <br>
	 * Has a constant value of <code>1</code>.
	 * @see		#drawImage(Object, int, int, int, boolean, int)
	 **/
	public static final int DRAW_LEFT_SIDE_ONLY		= 1;

	/**
	 * When drawing an image this value specifies that only the right hand side
	 * of the image should be drawn (used in Pillar Rising Mode). <br>
	 * <br>
	 * Has a constant value of <code>2</code>.
	 * @see		#drawImage(Object, int, int, int, boolean, int)
	 **/
	public static final int DRAW_RIGHT_SIDE_ONLY	= 2;



	/**
	 * This is called just before a <code>Layer</code> is drawn to allow the
	 * <code>Renderer</code> to set the desired clipping region. <br>
	 * @param objGFX the gfx object specific to the <code>Renderer</code>
	 * @param x the start coordinate of the clipping region
	 * @param y the start coordinate of the clipping region
	 * @param width	the width of the clipping region
	 * @param height the height of the clipping region
	 * @throws IllegalArgumentException if <code>objGFX</code> is <code>null</code>
	 **/
	public void setClip(Object objGFX, int x, int y, int width, int height) throws IllegalArgumentException;



	/**
	 * This is called after a <code>Layer</code> is drawn to restore the
	 * clipping region to what it used to be. <br>
	 * @param objGFX the gfx object specific to the <code>Renderer</code>
	 * @throws IllegalArgumentException if <code>objGFX</code> is <code>null</code>
	 **/
	public void restoreClip(Object objGFX) throws IllegalArgumentException;



	/**
	 * Draws an image as given by <code>imageIndex</code> at the given coordinates.
	 * @param objGFX the gfx object specific to the <code>Renderer</code>
	 * @param imageIndex the index of the image to draw - see {@link Map#getImageData()}
	 * @param dstPixX the coordinate of where the image should be rendered
	 * @param dstPixY the coordinate of where the image should be rendered
	 * @param transparency whether or not the image should rendered honouring
	 * it's transparent pixels.
	 * @param modification any modification style should be applied to the image
	 * @throws IllegalArgumentException if <code>objGFX</code> is <code>null</code>
	 */
	public void drawImage(Object objGFX, int imageIndex, int dstPixX, int dstPixY, boolean transparency, int modification) throws IllegalArgumentException;

}
