
import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Image;
import java.io.BufferedInputStream;
import java.io.IOException;

import com.alienfactory.javamappy.Layer;
import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.loader.MapLoader;
import com.alienfactory.javamappy.viewer.LayerViewer;
import com.alienfactory.javamappy.viewer.MapViewer;
import com.alienfactory.javamappy.viewer.render.JDK11Renderer;
import com.alienfactory.javamappy.viewer.render.Renderer;



//------------------------------------------------------------------------------
/**
 * <p>JavaMappy Example for Applets. </p>
 *
 * <p>This example loads the Alien Epidemic test map, starts a game loop in it's
 * own Thread and scrolls the map around the screen. </p>
 *
 * @author Steve Eynon of <a href="http://www.alienfactory.co.uk">Alien-Factory</a>.
 **/
//------------------------------------------------------------------------------
public class AppletExample extends Applet implements Runnable {

	private String		mapFileName		= "alienEpidemic.fmp";
	private int 		appletWidth 	= 320;		// applet dimensions
	private int 		appletHeight	= 256;
	private int 		speedX			= 2;		// no of pixels to move the map each frame
	private int 		speedY			= 2;
	private int 		refreshRate 	= 30;		// no of milli-seconds that should pass between each frame, 33 ~ 30Hz
	private	boolean		running 		= false;	// is set to true if the map is moving

	private Image 		doubleBuffer;
	private	MapViewer	mapViewer;
	private	LayerViewer	layerViewer;



	//--------------------------------------------------------------------------
	// JavaMappy Methods
	//--------------------------------------------------------------------------

	private void initialise() {
		BufferedInputStream inputStream = new BufferedInputStream(super.getClass().getResourceAsStream(mapFileName));

		// create the viewers, making them compatible with all Applets (JDK v1.1)
		Map map 	= MapLoader.loadMap(inputStream);
		Renderer r	= new JDK11Renderer(map);
		mapViewer 	= new MapViewer(map, r, appletWidth, appletHeight);
		layerViewer = mapViewer.getLayerViewers()[0];
		try { inputStream.close(); } catch (IOException ioe) { /**/ }

		// set some random initial coordinates
		layerViewer.setPixelX(10);
		layerViewer.setPixelY(15);

		// create double buffer
		doubleBuffer = super.createImage(appletWidth, appletHeight);
	}

	public void move() {
		int minX = 0, minY = 0;

		// adjust the max values to device dimensions
		Layer layer = layerViewer.getLayer();
		int maxX = layer.getWidthInPixels()  - super.getWidth();
		int maxY = layer.getHeightInPixels() - super.getHeight();

		layerViewer.addPixelX(speedX);
		if ((layerViewer.getPixelX() <= minX) ||
				(layerViewer.getPixelX() >= maxX)) {
			speedX = -speedX;
			layerViewer.addPixelX(speedX);
		}

		layerViewer.addPixelY(speedY);
		if ((layerViewer.getPixelY() <= minY) ||
				(layerViewer.getPixelY() >= maxY)) {
			speedY = -speedY;
			layerViewer.addPixelY(speedY);
		}
	}



	//----------------------------------------------------------------------
	// Implemented Runnable Methdos
	//----------------------------------------------------------------------

	public void run() {
		running = true;

		// the game loop
		while (running) {
			long startTime = System.currentTimeMillis();

			// scroll the map...
			move();

			// animate the map
			mapViewer.updateAnimBlocks();

			// ...and draw it in the buffer
			Graphics gfx = doubleBuffer.getGraphics();
			layerViewer.draw(gfx, false);
			gfx.dispose();

			// render the buffer to the screen
			super.repaint();

			// rest for the remainder of the jiffy
			long timeTaken = System.currentTimeMillis() - startTime;
			if (timeTaken < refreshRate) {
				try {
					// Sleep little one, sleep!
					Thread.sleep(refreshRate - timeTaken);
				} catch (InterruptedException ie) { /**/ }
			} else {
				// give other threads a chance to catch up
				Thread.yield();
			}
		}
	}



	//--------------------------------------------------------------------------
	// Overridden Applet Methods
	//--------------------------------------------------------------------------

	public void init() {
		// override default parameters with those passed in from the HTML
		appletWidth 	= getParamInt("Width", 		appletWidth);
		appletHeight	= getParamInt("Height", 	appletHeight);
		refreshRate		= getParamInt("RefreshRate",refreshRate);
		speedX 			= getParamInt("SpeedX", 	speedX);
		speedY 			= getParamInt("SpeedY", 	speedY);
		if (super.getParameter("MapFileName") != null)
			mapFileName = super.getParameter("MapFileName");

		initialise();
	}

	public void start() {
		// Go Mappy Go!!!
		Thread javaMappyExample = new Thread(this);
		javaMappyExample.start();
	}

	public void stop() {
		running = false;
	}

	public void destroy() {
		running = false;
		doubleBuffer.flush();
	}

	public void paint(Graphics gfx) {
		if (gfx == null) 			return;
		if (doubleBuffer == null)	return;
		gfx.drawImage(doubleBuffer, 0, 0, null);
	}

	public void update(Graphics gfx) {
		// this prevents the screen being white-washed
		paint(gfx);
	}

	public String getAppletInfo () {
		return "Applet for JavaMappy example [http://www.alienfactory.co.uk] [http://www.tilemap.co.uk]";
	}



	//--------------------------------------------------------------------------
	// Private Helper Methods
	//--------------------------------------------------------------------------

	/**
	 * Returns a value from the HTML Applet parameters. If the value was not set
	 * then the given default value is returned. <br>
	 **/
	private int getParamInt(String paramName, int defaultValue) {
		int value = defaultValue;

		try {
			String paramValue = super.getParameter(paramName);
			if (paramValue != null)
				value = Integer.parseInt(paramValue);
		} catch (NumberFormatException e) { /**/ }

		return value;
	}
}
