import java.io.IOException;
import java.io.InputStream;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Graphics;
import javax.microedition.midlet.MIDlet;

import com.alienfactory.javamappy.Layer;
import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.loader.MapLoader;
import com.alienfactory.javamappy.util.Logger;
import com.alienfactory.javamappy.viewer.LayerViewer;
import com.alienfactory.javamappy.viewer.MapViewer;
import com.alienfactory.javamappy.viewer.render.MIDP10Renderer;
import com.alienfactory.javamappy.viewer.render.Renderer;



//------------------------------------------------------------------------------
/**
 * <p>JavaMappy Example for MIDP 1.0 / J2ME Devices. </p>
 *
 * <p>This example loads the Alien Epidemic test map, starts a game loop in its
 * own Thread and scrolls the map around the screen. </p>
 *
 * @author Steve Eynon of <a href="http://www.alienfactory.co.uk">Alien-Factory</a>.
 **/
//------------------------------------------------------------------------------
public class MIDP10Example extends MIDlet {
	MappyScreen mappyScreen = new MappyScreen();



	// -------------------------------------------------------------------------
	// MIDlet Abstract Methods
	// -------------------------------------------------------------------------

	public void startApp() {
		// Go Mappy Go!!!
		mappyScreen.initialise();
		Display.getDisplay(this).setCurrent(mappyScreen);
		Thread javaMappyExample = new Thread(mappyScreen);
		javaMappyExample.start();
	}

	public void pauseApp() {
		mappyScreen.exit();
	}

	public void destroyApp(boolean unconditional) {
		mappyScreen.exit();
	}



	// -------------------------------------------------------------------------
	// JavaMappy Example
	// -------------------------------------------------------------------------

	public class MappyScreen extends Canvas implements Runnable {

		private final String		fmaFileName		= "/alienEpidemicJ2me.fma";
		private final String		pngFileName		= "/alienEpidemicJ2me.png";
		private int 				speedX			= 4;	// no of pixels to move the map each frame
		private int 				speedY			= 2;
		private final int			viewWidth		= super.getWidth();
		private final int			viewHeight		= super.getHeight();
		private MapViewer 			mapViewer;
		private LayerViewer			layerViewer;

		private static final int 	REFRESH_RATE 	= 30;	// no of milli-seconds that should pass between each frame, 33 ~ 30Hz
		private volatile boolean	running;				// is set to true if the map is moving



		// ---------------------------------------------------------------------
		// JavaMappy Methods
		// ---------------------------------------------------------------------

		private void initialise() {
			InputStream	inputStream = super.getClass().getResourceAsStream(fmaFileName);

			Logger.setLoggingEnabled(true);

			try {
				// create the viewers, making them compatible with J2ME / MIDP 1.0
				Map map		= MapLoader.loadMap(inputStream);
				Renderer r	= new MIDP10Renderer(map, pngFileName, true);
				mapViewer 	= new MapViewer(map, r, viewWidth, viewHeight);
				layerViewer = mapViewer.getLayerViewers()[0];
				inputStream.close();

			} catch (IOException ioe) {
				ioe.printStackTrace();
			}

			// set some random initial coordinates
			layerViewer.setPixelX(8);
			layerViewer.setPixelY(12);
		}

		private void move() {
			int minX = 0, minY = 0;

			// adjust the max values to device dimensions
			Layer layer = layerViewer.getLayer();
			int maxX = layer.getWidthInPixels()  - viewWidth;
			int maxY = layer.getHeightInPixels() - viewHeight;

			layerViewer.addPixelX(speedX);
			if ((layerViewer.getPixelX() <= minX) ||
					(layerViewer.getPixelX() >= maxX)) {
				speedX = -speedX;
				layerViewer.addPixelX(speedX);
			}

			layerViewer.addPixelY(speedY);
			if ((layerViewer.getPixelY() <= minY) ||
					(layerViewer.getPixelY() >= maxY)) {
				speedY = -speedY;
				layerViewer.addPixelY(speedY);
			}
		}

		public void exit() {
			running = false;
		}



		// ---------------------------------------------------------------------
		// Implemented Runnable Methdos
		// ---------------------------------------------------------------------

		public void run() {
			running = true;

			// the game loop
			while (running) {
				long startTime = System.currentTimeMillis();

				// scroll the map...
				move();

				// animate the map
				mapViewer.updateAnimBlocks();

				// ...and paint it on the screen
				super.repaint(0, 0, viewWidth, viewHeight);
				super.serviceRepaints();

				// rest for the remainder of the jiffy
				long timeTaken = System.currentTimeMillis() - startTime;
				if (timeTaken < REFRESH_RATE) {
					try {
						Thread.sleep(REFRESH_RATE - timeTaken);
					} catch (InterruptedException ie) { /**/ }
				} else {
					// give other threads a chance to catch up
					Thread.yield();
				}
			}
		}



		// ---------------------------------------------------------------------
		// Overridden Canvas Methods
		// ---------------------------------------------------------------------

		public void keyPressed(int keyCode) {
			// exit if a softkey is pressed
			int gameAction = super.getGameAction(keyCode);
			if (gameAction == Canvas.FIRE) {
				exit();
			}
		}

		public void paint(Graphics gfx) {
			layerViewer.drawBackground(gfx, false);
		}
	}
}
