import java.io.IOException;
import java.io.InputStream;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.GameCanvas;
import javax.microedition.lcdui.game.LayerManager;
import javax.microedition.midlet.MIDlet;

import com.alienfactory.javamappy.AnimBlock;
import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.loader.MapLoader;
import com.alienfactory.javamappy.viewer.TiledLayerWrapper;


//------------------------------------------------------------------------------
/**
 * <p>JavaMappy Example for MIDP 2.0 / J2ME Devices. </p>
 *
 * <p>This example loads the Alien Epidemic test map, starts a game loop in its
 * own Thread and scrolls the map around the screen. </p>
 *
 * <p>This example uses the MIDP 2.0 TiledLayer and LayerManager classes.</p>
 *
 * @author Steve Eynon of <a href="http://www.alienfactory.co.uk/">Alien-Factory</a>.
 **/
//------------------------------------------------------------------------------
public class MIDP20Example extends MIDlet {
	MappyScreen mappyScreen = new MappyScreen();



	// -------------------------------------------------------------------------
	// MIDlet Abstract Methods
	// -------------------------------------------------------------------------

	public void startApp() {
		// Go Mappy Go!!!
		mappyScreen.initialise();
		Display.getDisplay(this).setCurrent(mappyScreen);
		Thread javaMappyExample = new Thread(mappyScreen);
		javaMappyExample.start();
	}

	public void pauseApp() {
		mappyScreen.exit();
	}

	public void destroyApp(boolean unconditional) {
		mappyScreen.exit();
	}



	// -------------------------------------------------------------------------
	// JavaMappy Example
	// -------------------------------------------------------------------------

	public class MappyScreen extends GameCanvas implements Runnable {

		private final String		fmaFileName		= "/alienEpidemicJ2me.fma";
		private final String		pngFileName		= "/alienEpidemicJ2me.png";
		private int 				speedX			= 2;	// no of pixels to move the map each frame
		private int 				speedY			= 4;
		private final int			viewWidth		= super.getWidth();
		private final int			viewHeight		= super.getHeight();
		private LayerManager		layerManager	= new LayerManager();
		private TiledLayerWrapper	tiledLayerWrapper;
		private int					x = 8, y = 12;

		private static final int 	REFRESH_RATE 	= 30;	// no of milli-seconds that should pass between each frame, 33 ~ 30Hz
		private volatile boolean	running;				// is set to true if the map is moving

		public MappyScreen() {
			super(false);
		}

		// ---------------------------------------------------------------------
		// JavaMappy Methods
		// ---------------------------------------------------------------------

		private void initialise() {
			InputStream	inputStream = super.getClass().getResourceAsStream(fmaFileName);

			try {
				// create the viewers, making them compatible with J2ME / MIDP 2.0
				Map map	= MapLoader.loadMap(inputStream);
				Image tileSheet = Image.createImage(pngFileName);
				tiledLayerWrapper = new TiledLayerWrapper(map.getLayers()[0], 0, tileSheet, true);
				layerManager.append(tiledLayerWrapper.getTiledLayer());
				inputStream.close();

			} catch (IOException ioe) {
				ioe.printStackTrace();
				throw new RuntimeException(ioe.getMessage());
			}
		}

		private void move() {
			int minX = 0, minY = 0;

			// adjust the max values to device dimensions
			int maxX = tiledLayerWrapper.getLayer().getWidthInPixels()  - viewWidth;
			int maxY = tiledLayerWrapper.getLayer().getHeightInPixels() - viewHeight;

			x += speedX;
			if ((x <= minX) || (x >= maxX)) {
				speedX = -speedX;
				x += speedX;
			}

			y += speedY;
			if ((y <= minY) || (y >= maxY)) {
				speedY = -speedY;
				y += speedY;
			}
		}

		public void exit() {
			running = false;
		}



		// ---------------------------------------------------------------------
		// Implemented Runnable Methods :: The Game Loop
		// ---------------------------------------------------------------------

		public void run() {
			running = true;

			 // Get the Graphics object for the off-screen buffer
			 Graphics g = getGraphics();

			// the game loop
			while (running) {
				long startTime = System.currentTimeMillis();

				// scroll the map...
				move();

				// animate the blocks
				AnimBlock[] animBlocks = tiledLayerWrapper.getLayer().getAnimBlocks();
				for (int index = 0; index < animBlocks.length; index++)
					animBlocks[index].updateAnimation();

				// animate the layer
				tiledLayerWrapper.animateLayer();

				// repaint the map
				// NOTE: the coordinates are the inverse of JavaMappy's
				g.setColor(0x00FF00);
				g.fillRect(0, 0, viewWidth, viewHeight);
				layerManager.paint(g, -x, -y);

				// flush the off-screen buffer to the device
				flushGraphics();

				// rest for the remainder of the jiffy
				long timeTaken = System.currentTimeMillis() - startTime;
				if (timeTaken < REFRESH_RATE) {
					try {
						Thread.sleep(REFRESH_RATE - timeTaken);
					} catch (InterruptedException ie) { /**/ }
				} else {
					// give other threads a chance to catch up
					Thread.yield();
				}
			}
		}



		// ---------------------------------------------------------------------
		// Overridden Canvas Methods
		// ---------------------------------------------------------------------

		public void keyPressed(int keyCode) {
			// exit if a softkey is pressed
			int gameAction = super.getGameAction(keyCode);
			if (gameAction == Canvas.FIRE) {
				exit();
			}
		}
	}
}
