import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;

import javax.swing.JFrame;

import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.loader.MapLoader;
import com.alienfactory.javamappy.util.Logger;
import com.alienfactory.javamappy.viewer.LayerViewer;
import com.alienfactory.javamappy.viewer.MapViewer;
import com.alienfactory.javamappy.viewer.render.J2SE14Renderer;
import com.alienfactory.javamappy.viewer.render.JDK11Renderer;
import com.alienfactory.javamappy.viewer.render.JDK12Renderer;
import com.alienfactory.javamappy.viewer.render.Renderer;



//------------------------------------------------------------------------------
/**
 * <p>JavaMappy Example for Swing. </p>
 * 
 * <p>This example loads the Isometric test map, starts a game loop in it's 
 * own Thread and lets you scroll the map around using the cursor keys. </p>
 * 
 * @author Slimer of <a href="http://www.alienfactory.co.uk">Alien-Factory</a>.
 **/
//------------------------------------------------------------------------------
public class SwingExample extends JFrame implements Runnable {
	
	private static String	mapFileName		= "isometric.fmp";
	private static int 		windowWidth 	= 320;		// dimensions in pixels
	private static int 		windowHeight	= 256;
	private static int 		speedX			= 2;		// no of pixels to move the map each frame
	private static int 		speedY			= 2;
	private static int 		refreshRate 	= 30;		// no of milli-seconds that should pass between each frame, 33 ~ 30Hz
	private	static boolean	running 		= false;	// is set to true if the map is moving
	
	private Image 			doubleBuffer;
	private	MapViewer		mapViewer;
	private	LayerViewer		layerViewer;
	private KeyboardInput	keyboardInput	= new KeyboardInput();
	private static Class 	renderType		= JDK12Renderer.class;
	
	
	
	// -------------------------------------------------------------------------
	// JavaMappy Methods
	// -------------------------------------------------------------------------
	
	private void initialise() throws Exception {
		BufferedInputStream inputStream = new BufferedInputStream(new FileInputStream(mapFileName));
		
		// create the viewers, making them render with the given JDK
		Map map 	= MapLoader.loadMap(inputStream);
		Renderer r	= ((Renderer) renderType.getConstructor(new Class[] {Map.class}).newInstance(new Object[] {map}));
		mapViewer 	= new MapViewer(map, r, windowWidth, windowHeight, 0, 0);
		layerViewer = mapViewer.getLayerViewers()[0];
		try { inputStream.close(); } catch (IOException ioe) { /**/ }
		
		// create double buffer
		doubleBuffer = super.createImage(windowWidth, windowHeight);
	}
	
	private void move() {
		int minX = 0, minY = 0;
		int maxX = layerViewer.getLayer().getWidthInPixels() - windowWidth;
		int maxY = layerViewer.getLayer().getHeightInPixels() - windowHeight;
		
		// move map
		if (keyboardInput.LEFT) 	layerViewer.addPixelX(-speedX);
		if (keyboardInput.RIGHT)	layerViewer.addPixelX( speedX);
		if (keyboardInput.UP)  		layerViewer.addPixelY(-speedY);
		if (keyboardInput.DOWN)		layerViewer.addPixelY( speedY);
		
		// keep map within coordinate constraints
		if (layerViewer.getPixelX() < minX)	layerViewer.setPixelX(minX);
		if (layerViewer.getPixelX() > maxX)	layerViewer.setPixelX(maxX);
		if (layerViewer.getPixelY() < minY)	layerViewer.setPixelY(minY);
		if (layerViewer.getPixelY() > maxY)	layerViewer.setPixelY(maxY);		
	}
	
	
	
	// -------------------------------------------------------------------------
	// Implemented Runnable Methdos
	// -------------------------------------------------------------------------
	
	public void run() {
		running = true;
		
		// the game loop
		while (running) {
			long startTime = System.currentTimeMillis();
			
			// scroll the map...
			move();
			
			// animate the map...
			mapViewer.updateAnimBlocks();
			
			// ...and draw it in the buffer
			Graphics gfx = doubleBuffer.getGraphics();			
			layerViewer.draw(gfx, false);
			gfx.setColor(Color.RED);
			gfx.drawString("X: " + layerViewer.getPixelX() + "     Y: " + layerViewer.getPixelY(), 10, 40);
			gfx.dispose();
			
			// render the buffer to the screen
			super.repaint();
			
			// rest for the remainder of the jiffy
			long timeTaken = System.currentTimeMillis() - startTime;
			if (timeTaken < refreshRate) {
				try { 
					// Sleep little one, sleep!
					Thread.sleep(refreshRate - timeTaken);
				} catch (InterruptedException ie) { /**/ }
			} else {
				// give other threads a chance to catch up
				Thread.yield();
			}
		}
	}
	
	
	
	// -------------------------------------------------------------------------
	// Overridden JFrame Methods
	// -------------------------------------------------------------------------
	
	public void paint(Graphics g) {
		if (doubleBuffer != null) {
			// take into account the window border 
			Insets insets = super.getInsets();
			g.drawImage(doubleBuffer, insets.left, insets.top, null);
		}
	}
	
	public void update(Graphics g) {
		// this prevents the screen being white-washed
		paint(g);
	}
	
	

	// -------------------------------------------------------------------------
	// The Main Method
	// -------------------------------------------------------------------------
	
	public static void main(String[] args) throws Exception {
		Logger.setLoggingEnabled(true);		
		
		// override default parameters with those passed in from the command line
		System.out.println("Usage: java SwingExample JDK11|JDK13|J2SE14 filename.fmp");
		if (args.length > 0) {
			if (args[0].equals("JDK11"))
				renderType = JDK11Renderer.class;
			if (args[0].equals("JDK12"))
				renderType = JDK12Renderer.class;
			if (args[0].equals("J2SE14"))
				renderType = J2SE14Renderer.class;
		}
		if (args.length > 1)
			mapFileName = args[1];        
		
		// use the Swing API to display the map
		SwingExample window = new SwingExample();
		window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		window.addKeyListener(window.keyboardInput);
		window.setTitle("JavaMappy :: Use cursor keys");
		window.setVisible(true);

		// adjest the size of the window to take into account the window border 
		Insets insets = window.getInsets();
		window.setSize(insets.left + windowWidth + insets.right, insets.top + windowHeight + insets.bottom);
		
		// Go Mappy Go!!!
		window.initialise();
		new Thread(window).start();
	}

}
